import re

from typing import Pattern
from unittest import mock

import pytest

from faker import Faker, providers
from faker.providers.address.az_AZ import Provider as AzAzAddressProvider
from faker.providers.address.cs_CZ import Provider as CsCzAddressProvider
from faker.providers.address.da_DK import Provider as DaDkAddressProvider
from faker.providers.address.de_AT import Provider as DeAtAddressProvider
from faker.providers.address.de_CH import Provider as DeChAddressProvider
from faker.providers.address.de_DE import Provider as DeDeAddressProvider
from faker.providers.address.el_GR import Provider as ElGrAddressProvider
from faker.providers.address.en_AU import Provider as EnAuAddressProvider
from faker.providers.address.en_BD import Provider as EnBdAddressProvider
from faker.providers.address.en_CA import Provider as EnCaAddressProvider
from faker.providers.address.en_GB import Provider as EnGbAddressProvider
from faker.providers.address.en_IE import Provider as EnIeAddressProvider
from faker.providers.address.en_IN import Provider as EnInAddressProvider
from faker.providers.address.en_NZ import Provider as EnNzAddressProvider
from faker.providers.address.en_PH import Provider as EnPhAddressProvider
from faker.providers.address.en_US import Provider as EnUsAddressProvider
from faker.providers.address.es_CO import Provider as EsCoAddressProvider
from faker.providers.address.es_ES import Provider as EsEsAddressProvider
from faker.providers.address.es_MX import Provider as EsMxAddressProvider
from faker.providers.address.fa_IR import Provider as FaIrAddressProvider
from faker.providers.address.fi_FI import Provider as FiFiAddressProvider
from faker.providers.address.fr_CA import Provider as FrCaAddressProvider
from faker.providers.address.fr_CH import Provider as FrChAddressProvider
from faker.providers.address.fr_FR import Provider as FrFrAddressProvider
from faker.providers.address.he_IL import Provider as HeIlAddressProvider
from faker.providers.address.hi_IN import Provider as HiInAddressProvider
from faker.providers.address.hr_HR import Provider as HrHrAddressProvider
from faker.providers.address.hu_HU import Provider as HuHuAddressProvider
from faker.providers.address.hy_AM import Provider as HyAmAddressProvider
from faker.providers.address.id_ID import Provider as IdIdAddressProvider
from faker.providers.address.it_IT import Provider as ItItAddressProvider
from faker.providers.address.ja_JP import Provider as JaJpAddressProvider
from faker.providers.address.ka_GE import Provider as KaGeAddressProvider
from faker.providers.address.ko_KR import Provider as KoKrAddressProvider
from faker.providers.address.ne_NP import Provider as NeNpAddressProvider
from faker.providers.address.no_NO import Provider as NoNoAddressProvider
from faker.providers.address.pt_BR import Provider as PtBrAddressProvider
from faker.providers.address.pt_PT import Provider as PtPtAddressProvider
from faker.providers.address.ro_RO import Provider as RoRoAddressProvider
from faker.providers.address.ru_RU import Provider as RuRuAddressProvider
from faker.providers.address.sk_SK import Provider as SkSkAddressProvider
from faker.providers.address.sl_SI import Provider as SlSiAddressProvider
from faker.providers.address.sv_SE import Provider as SvSeAddressProvider
from faker.providers.address.ta_IN import Provider as TaInAddressProvider
from faker.providers.address.th_TH import Provider as ThThAddressProvider
from faker.providers.address.uk_UA import Provider as UkUaAddressProvider
from faker.providers.address.zh_CN import Provider as ZhCnAddressProvider
from faker.providers.address.zh_TW import Provider as ZhTwAddressProvider


class TestBaseProvider:
    """Test address provider methods"""

    def test_alpha_2_country_codes(self, faker, num_samples):
        for _ in range(num_samples):
            country_code = faker.country_code(representation="alpha-2")
            assert len(country_code) == 2
            assert country_code.isalpha()

    def test_alpha_2_country_codes_as_default(self, faker, num_samples):
        for _ in range(num_samples):
            country_code = faker.country_code()
            assert len(country_code) == 2
            assert country_code.isalpha()

    def test_alpha_3_country_codes(self, faker, num_samples):
        for _ in range(num_samples):
            country_code = faker.country_code(representation="alpha-3")
            assert len(country_code) == 3
            assert country_code.isalpha()

    def test_bad_country_code_representation(self, faker, num_samples):
        for _ in range(num_samples):
            with pytest.raises(ValueError):
                faker.country_code(representation="hello")

    def _collect_fakers_for_locales(self):
        cached_locales = []
        language_locale_codes = providers.BaseProvider.language_locale_codes
        for code, countries in language_locale_codes.items():
            for country in countries:
                name = f"{code}_{country}"
                try:
                    faker = Faker(name)
                    cached_locales.append(faker)
                except AttributeError as e:
                    print(f"Cannot generate faker for {name}: {e}. Skipped")

        return cached_locales

    def _fakers_for_locales(self):
        if not hasattr(self.__class__, "cached_locales"):
            self.__class__.cached_locales = self._collect_fakers_for_locales()
        return self.cached_locales

    def test_administrative_unit_all_locales(self):
        for faker in self._fakers_for_locales():
            if faker.current_country_code() not in ["IL", "GE", "TW", "UA", "NZ"]:
                try:
                    assert isinstance(faker.administrative_unit(), str)
                except Exception as e:
                    raise e.__class__(faker.current_country_code(), *e.args)

    def test_country_code_all_locales(self):
        for faker in self._fakers_for_locales():
            assert isinstance(faker.current_country(), str)

    def test_current_country_errors(self):
        dt = providers.date_time
        countries_duplicated = [*dt.Provider.countries, *dt.Provider.countries]
        with mock.patch.object(dt.Provider, "countries", countries_duplicated), pytest.raises(ValueError) as e:
            Faker("en_US").current_country()
        assert "Ambiguous" in str(e)
        country_code = "faker.providers.address.Provider.current_country_code"
        with pytest.raises(ValueError), mock.patch(country_code, lambda self: "en_ZZ"):
            Faker("en_US").current_country()


class TestAzAz:
    """Test az_AZ address provider methods"""

    def test_street_suffix_long(self, faker, num_samples):
        for _ in range(num_samples):
            street_suffix_long = faker.street_suffix()
            assert isinstance(street_suffix_long, str)
            assert street_suffix_long in AzAzAddressProvider.street_suffixes

    def test_city_name(self, faker, num_samples):
        for _ in range(num_samples):
            city = faker.city()
            assert isinstance(city, str)
            assert city in AzAzAddressProvider.cities

    def test_street_name(self, faker, num_samples):
        for _ in range(num_samples):
            street_name = faker.street()
            assert isinstance(street_name, str)
            assert street_name in AzAzAddressProvider.streets

    def test_settlement_name(self, faker, num_samples):
        for _ in range(num_samples):
            settlement_name = faker.settlement()
            assert isinstance(settlement_name, str)
            assert settlement_name in AzAzAddressProvider.settlements

    def test_village_name(self, faker, num_samples):
        for _ in range(num_samples):
            village_name = faker.village()
            assert isinstance(village_name, str)
            assert village_name in AzAzAddressProvider.villages

    def test_postcode(self, faker, num_samples):
        for _ in range(num_samples):
            postcode = faker.postcode()
            assert isinstance(postcode, str)
            assert re.fullmatch(r"AZ\d{4}", postcode)
            assert int(postcode[2:]) in range(900, 6600)


class TestCsCz:
    """Test cs_CZ address provider methods"""

    def test_street_suffix_short(self, faker, num_samples):
        for _ in range(num_samples):
            street_suffix_short = faker.street_suffix_short()
            assert isinstance(street_suffix_short, str)
            assert street_suffix_short in CsCzAddressProvider.street_suffixes_short

    def test_street_suffix_long(self, faker, num_samples):
        for _ in range(num_samples):
            street_suffix_long = faker.street_suffix_long()
            assert isinstance(street_suffix_long, str)
            assert street_suffix_long in CsCzAddressProvider.street_suffixes_long

    def test_city_name(self, faker, num_samples):
        for _ in range(num_samples):
            city = faker.city_name()
            assert isinstance(city, str)
            assert city in CsCzAddressProvider.cities

    def test_street_name(self, faker, num_samples):
        for _ in range(num_samples):
            street_name = faker.street_name()
            assert isinstance(street_name, str)
            assert street_name in CsCzAddressProvider.streets

    def test_state(self, faker, num_samples):
        for _ in range(num_samples):
            state = faker.state()
            assert isinstance(state, str)
            assert state in CsCzAddressProvider.states

    def test_postcode(self, faker, num_samples):
        for _ in range(num_samples):
            postcode = faker.postcode()
            assert isinstance(postcode, str)
            assert re.fullmatch(r"\d{3} \d{2}", postcode)

    def test_city_with_postcode(self, faker, num_samples):
        for _ in range(num_samples):
            city_with_postcode = faker.city_with_postcode()
            assert isinstance(city_with_postcode, str)
            match = re.fullmatch(r"\d{3} \d{2} (?P<city>.*)", city_with_postcode)
            assert match.group("city") in CsCzAddressProvider.cities


class TestDaDk:
    """Test dk_DK address provider methods"""

    def test_street_suffix(self, faker, num_samples):
        for _ in range(num_samples):
            street_suffix = faker.street_suffix()
            assert isinstance(street_suffix, str)
            assert street_suffix in DaDkAddressProvider.street_suffixes

    def test_street_name(self, faker, num_samples):
        for _ in range(num_samples):
            street_name = faker.street_name()
            assert isinstance(street_name, str)

    def test_dk_street_name(self, faker, num_samples):
        for _ in range(num_samples):
            dk_street_name = faker.dk_street_name()
            assert isinstance(dk_street_name, str)

    def test_city_name(self, faker, num_samples):
        for _ in range(num_samples):
            city = faker.city_name()
            assert isinstance(city, str)
            assert city in DaDkAddressProvider.cities

    def test_state(self, faker, num_samples):
        for _ in range(num_samples):
            state = faker.state()
            assert isinstance(state, str)
            assert state in DaDkAddressProvider.states

    def test_postcode(self, faker, num_samples):
        for _ in range(num_samples):
            postcode = faker.postcode()
            assert isinstance(postcode, str)
            assert re.fullmatch(r"\d{4}", postcode)


class TestDeAt:
    """Test de_AT address provider methods"""

    def test_city(self, faker, num_samples):
        for _ in range(num_samples):
            city = faker.city()
            assert isinstance(city, str)
            assert city in DeAtAddressProvider.cities

    def test_state(self, faker, num_samples):
        for _ in range(num_samples):
            state = faker.state()
            assert isinstance(state, str)
            assert state in DeAtAddressProvider.states

    def test_street_suffix_short(self, faker, num_samples):
        for _ in range(num_samples):
            street_suffix_short = faker.street_suffix_short()
            assert isinstance(street_suffix_short, str)
            assert street_suffix_short in DeAtAddressProvider.street_suffixes_short

    def test_street_suffix_long(self, faker, num_samples):
        for _ in range(num_samples):
            street_suffix_long = faker.street_suffix_long()
            assert isinstance(street_suffix_long, str)
            assert street_suffix_long in DeAtAddressProvider.street_suffixes_long

    def test_country(self, faker, num_samples):
        for _ in range(num_samples):
            country = faker.country()
            assert isinstance(country, str)
            assert country in DeAtAddressProvider.countries

    def test_postcode(self, faker, num_samples):
        for _ in range(num_samples):
            postcode = faker.postcode()
            assert isinstance(postcode, str)
            assert re.fullmatch(r"\d{4}", postcode)

    def test_city_with_postcode(self, faker, num_samples):
        for _ in range(num_samples):
            city_with_postcode = faker.city_with_postcode()
            assert isinstance(city_with_postcode, str)
            match = re.fullmatch(r"\d{4} (?P<city>.*)", city_with_postcode)
            assert match.groupdict()["city"] in DeAtAddressProvider.cities


class TestDeDe:
    """Test de_DE address provider methods"""

    def test_city(self, faker, num_samples):
        for _ in range(num_samples):
            city = faker.city()
            assert isinstance(city, str)
            assert city in DeDeAddressProvider.cities

    def test_state(self, faker, num_samples):
        for _ in range(num_samples):
            state = faker.state()
            assert isinstance(state, str)
            assert state in DeDeAddressProvider.states

    def test_street_suffix_short(self, faker, num_samples):
        for _ in range(num_samples):
            street_suffix_short = faker.street_suffix_short()
            assert isinstance(street_suffix_short, str)
            assert street_suffix_short in DeDeAddressProvider.street_suffixes_short

    def test_street_suffix_long(self, faker, num_samples):
        for _ in range(num_samples):
            street_suffix_long = faker.street_suffix_long()
            assert isinstance(street_suffix_long, str)
            assert street_suffix_long in DeDeAddressProvider.street_suffixes_long

    def test_country(self, faker, num_samples):
        for _ in range(num_samples):
            country = faker.country()
            assert isinstance(country, str)
            assert country in DeDeAddressProvider.countries

    def test_postcode(self, faker, num_samples):
        for _ in range(num_samples):
            postcode = faker.postcode()
            assert isinstance(postcode, str)
            assert re.fullmatch(r"\d{5}", postcode)

    def test_city_with_postcode(self, faker, num_samples):
        for _ in range(num_samples):
            city_with_postcode = faker.city_with_postcode()
            assert isinstance(city_with_postcode, str)
            match = re.fullmatch(r"\d{5} (?P<city>.*)", city_with_postcode)
            assert match.groupdict()["city"] in DeDeAddressProvider.cities


class TestElGr:
    """Test el_GR address provider methods"""

    def test_line_address(self, faker, num_samples):
        for _ in range(num_samples):
            address = faker.line_address()
            assert isinstance(address, str)

    def test_street_prefix_short(self, faker, num_samples):
        for _ in range(num_samples):
            street_prefix_short = faker.street_prefix_short()
            assert isinstance(street_prefix_short, str)
            assert street_prefix_short in ElGrAddressProvider.street_prefixes_short

    def test_street_prefix_long(self, faker, num_samples):
        for _ in range(num_samples):
            street_prefix_long = faker.street_prefix_long()
            assert isinstance(street_prefix_long, str)
            assert street_prefix_long in ElGrAddressProvider.street_prefixes_long

    def test_street(self, faker, num_samples):
        for _ in range(num_samples):
            street = faker.street()
            assert isinstance(street, str)
            assert street in ElGrAddressProvider.localities

    def test_city(self, faker, num_samples):
        for _ in range(num_samples):
            city = faker.city()
            assert isinstance(city, str)
            assert city in ElGrAddressProvider.cities

    def test_region(self, faker, num_samples):
        for _ in range(num_samples):
            region = faker.region()
            assert isinstance(region, str)
            assert region in ElGrAddressProvider.regions


class TestEnAu:
    """Test en_AU address provider methods"""

    def test_postcode(self, faker, num_samples):
        for _ in range(num_samples):
            postcode = faker.postcode()
            assert isinstance(postcode, str)
            assert re.fullmatch(r"\d{4}", postcode)

    def test_state(self, faker, num_samples):
        for _ in range(num_samples):
            state = faker.state()
            assert isinstance(state, str)
            assert state in EnAuAddressProvider.states

    def test_city_prefix(self, faker, num_samples):
        for _ in range(num_samples):
            city_prefix = faker.city_prefix()
            assert isinstance(city_prefix, str)
            assert city_prefix in EnAuAddressProvider.city_prefixes

    def test_state_abbr(self, faker, num_samples):
        for _ in range(num_samples):
            state_abbr = faker.state_abbr()
            assert isinstance(state_abbr, str)
            assert state_abbr in EnAuAddressProvider.states_abbr
            assert state_abbr.isupper()


class TestEnBd:
    """Test en_BD address provider methods"""

    def test_administrative_unit(self, faker, num_samples):
        for _ in range(num_samples):
            administrative_unit = faker.administrative_unit()
            assert isinstance(administrative_unit, str)
            assert administrative_unit in EnBdAddressProvider.cities

    def test_area_name(self, faker, num_samples):
        for _ in range(num_samples):
            area_name = faker.area_name()
            assert isinstance(area_name, str)
            assert area_name in EnBdAddressProvider.area_names

    def test_building_name(self, faker, num_samples):
        for _ in range(num_samples):
            building_name = faker.building_name()
            assert isinstance(building_name, str)
            assert building_name in EnBdAddressProvider.building_names

    def test_building_number(self, faker, num_samples):
        for _ in range(num_samples):
            building_number = faker.building_number()
            assert isinstance(building_number, str)

    def test_city_prefix(self, faker, num_samples):
        for _ in range(num_samples):
            city_prefix = faker.city_prefix()
            assert isinstance(city_prefix, str)
            assert city_prefix in EnBdAddressProvider.city_prefixes

    def test_city(self, faker, num_samples):
        for _ in range(num_samples):
            city = faker.city()
            assert isinstance(city, str)
            assert city in EnBdAddressProvider.cities

    def test_postcode(self, faker, num_samples):
        for _ in range(num_samples):
            postcode = faker.postcode()
            assert isinstance(postcode, str)
            assert re.fullmatch(r"\d{4}", postcode)

    def test_secondary_address(self, faker, num_samples):
        for _ in range(num_samples):
            secondary_address = faker.secondary_address()
            assert isinstance(secondary_address, str)

    def test_town(self, faker, num_samples):
        for _ in range(num_samples):
            town = faker.town()
            assert isinstance(town, str)


class TestEnCa:
    """Test en_CA address provider methods"""

    valid_postcode_letter_re = r"[{}]".format("".join(EnCaAddressProvider.postal_code_letters))
    valid_postcode_re = r"{0}[0-9]{0} ?[0-9]{0}[0-9]".format(valid_postcode_letter_re)

    def test_postcode(self, faker, num_samples):
        for _ in range(num_samples):
            postcode = faker.postcode()
            assert isinstance(postcode, str)
            assert re.fullmatch(self.valid_postcode_re, postcode)

    def test_postcode_in_province(self, faker, num_samples):
        for _ in range(num_samples):
            for province_abbr in EnCaAddressProvider.provinces_abbr:
                code = faker.postcode_in_province(province_abbr)
                assert code[0] in EnCaAddressProvider.provinces_postcode_prefixes[province_abbr]
                with pytest.raises(Exception):
                    faker.postcode_in_province("XX")

    def test_postalcode(self, faker, num_samples):
        for _ in range(num_samples):
            postalcode = faker.postalcode()
            assert isinstance(postalcode, str)
            assert re.fullmatch(self.valid_postcode_re, postalcode)

    def test_postal_code_letter(self, faker, num_samples):
        for _ in range(num_samples):
            postal_code_letter = faker.postal_code_letter()
            assert isinstance(postal_code_letter, str)
            assert re.fullmatch(self.valid_postcode_letter_re, postal_code_letter)

    def test_province(self, faker, num_samples):
        for _ in range(num_samples):
            province = faker.province()
            assert isinstance(province, str)
            assert province in EnCaAddressProvider.provinces

    def test_province_abbr(self, faker, num_samples):
        for _ in range(num_samples):
            province_abbr = faker.province_abbr()
            assert isinstance(province_abbr, str)
            assert province_abbr in EnCaAddressProvider.provinces_abbr

    def test_city_prefix(self, faker, num_samples):
        for _ in range(num_samples):
            city_prefix = faker.city_prefix()
            assert isinstance(city_prefix, str)
            assert city_prefix in EnCaAddressProvider.city_prefixes

    def test_secondary_address(self, faker, num_samples):
        for _ in range(num_samples):
            secondary_address = faker.secondary_address()
            assert isinstance(secondary_address, str)
            assert re.fullmatch(r"(?:Apt\.|Suite) \d{3}", secondary_address)


class TestEnGb:
    """Test en_GB address provider methods"""

    def test_postcode(self, faker, num_samples):
        ukpcp = pytest.importorskip("ukpostcodeparser.parser")
        for _ in range(num_samples):
            assert isinstance(ukpcp.parse_uk_postcode(faker.postcode()), tuple)

    def test_county(self, faker, num_samples):
        for _ in range(num_samples):
            county = faker.county()
            assert isinstance(county, str)
            assert county in EnGbAddressProvider.counties


class TestEnIe:
    """Test en_IE address provider methods"""

    def test_postcode(self, faker, num_samples):
        """https://stackoverflow.com/questions/33391412/validation-for-irish-eircode"""
        for _ in range(num_samples):
            postcode = faker.postcode()
            assert isinstance(postcode, str)
            assert re.fullmatch(r"(?:^[AC-FHKNPRTV-Y][0-9]{2}|D6W)[ -]?[0-9AC-FHKNPRTV-Y]{4}$", postcode)

    def test_county(self, faker, num_samples):
        for _ in range(num_samples):
            county = faker.county()
            assert isinstance(county, str)
            assert county in EnIeAddressProvider.counties


class TestEnUS:
    """Test en_US address provider methods"""

    def test_city_prefix(self, faker, num_samples):
        for _ in range(num_samples):
            city_prefix = faker.city_prefix()
            assert isinstance(city_prefix, str)
            assert city_prefix in EnUsAddressProvider.city_prefixes

    def test_state(self, faker, num_samples):
        for _ in range(num_samples):
            state = faker.state()
            assert isinstance(state, str)
            assert state in EnUsAddressProvider.states

    def test_state_abbr(self, faker, num_samples):
        for _ in range(num_samples):
            state_abbr = faker.state_abbr()
            assert isinstance(state_abbr, str)
            states_and_territories = EnUsAddressProvider.known_usps_abbr
            assert state_abbr in states_and_territories

    def test_state_abbr_states_only(self, faker, num_samples):
        for _ in range(num_samples):
            state_abbr = faker.state_abbr(include_territories=False, include_freely_associated_states=False)
            assert isinstance(state_abbr, str)
            assert state_abbr in EnUsAddressProvider.states_abbr

    def test_state_abbr_no_territories(self, faker, num_samples):
        for _ in range(num_samples):
            state_abbr = faker.state_abbr(include_territories=False)
            assert isinstance(state_abbr, str)
            assert (
                state_abbr in EnUsAddressProvider.states_abbr
                or state_abbr in EnUsAddressProvider.freely_associated_states_abbr
            )

    def test_state_abbr_no_freely_associated_states(self, faker, num_samples):
        for _ in range(num_samples):
            state_abbr = faker.state_abbr(include_freely_associated_states=False)
            assert isinstance(state_abbr, str)
            assert state_abbr in EnUsAddressProvider.states_abbr or state_abbr in EnUsAddressProvider.territories_abbr

    def test_postcode(self, faker, num_samples):
        for _ in range(num_samples):
            code = faker.postcode()
            assert isinstance(code, str) and len(code) == 5
            assert 501 <= int(code) <= 99950

    def test_postcode_in_state(self, faker, num_samples):
        for _ in range(num_samples):
            for state_abbr in EnUsAddressProvider.known_usps_abbr:
                code = faker.postcode_in_state(state_abbr)
                assert re.fullmatch(r"\d{5}", code)
                assert int(code) >= EnUsAddressProvider.states_postcode[state_abbr][0]
                assert int(code) <= EnUsAddressProvider.states_postcode[state_abbr][1]

        with pytest.raises(Exception):
            faker.postcode_in_state("XX")

    def test_zipcode(self, faker, num_samples):
        for _ in range(num_samples):
            zipcode = faker.zipcode()
            assert isinstance(zipcode, str) and len(zipcode) == 5
            assert 501 <= int(zipcode) <= 99950

    def test_zipcode_in_state(self, faker, num_samples):
        for _ in range(num_samples):
            for state_abbr in EnUsAddressProvider.known_usps_abbr:
                code = faker.zipcode_in_state(state_abbr)
                assert re.fullmatch(r"\d{5}", code)
                assert int(code) >= EnUsAddressProvider.states_postcode[state_abbr][0]
                assert int(code) <= EnUsAddressProvider.states_postcode[state_abbr][1]

        with pytest.raises(Exception):
            faker.zipcode_in_state("XX")

    def test_zipcode_plus4(self, faker, num_samples):
        for _ in range(num_samples):
            zipcode_plus4 = faker.zipcode_plus4()
            assert isinstance(zipcode_plus4, str)
            zipcode, plus4 = zipcode_plus4.split("-")
            assert 501 <= int(zipcode) <= 99950
            assert 1 <= int(plus4) <= 9999

    def test_military_ship(self, faker, num_samples):
        for _ in range(num_samples):
            military_ship = faker.military_ship()
            assert isinstance(military_ship, str)
            assert military_ship in EnUsAddressProvider.military_ship_prefix

    def test_military_state(self, faker, num_samples):
        for _ in range(num_samples):
            military_state = faker.military_state()
            assert isinstance(military_state, str)
            assert military_state in EnUsAddressProvider.military_state_abbr

    def test_military_apo(self, faker, num_samples):
        for _ in range(num_samples):
            military_apo = faker.military_apo()
            assert isinstance(military_apo, str)
            assert re.fullmatch(r"PSC \d{4}, Box \d{4}", military_apo)

    def test_military_dpo(self, faker, num_samples):
        for _ in range(num_samples):
            military_dpo = faker.military_dpo()
            assert isinstance(military_dpo, str)
            assert re.fullmatch(r"Unit \d{4} Box \d{4}", military_dpo)

    def test_postalcode(self, faker, num_samples):
        for _ in range(num_samples):
            postalcode = faker.postalcode()
            assert isinstance(postalcode, str) and len(postalcode) == 5
            assert 501 <= int(postalcode) <= 99950

    def test_postalcode_in_state(self, faker, num_samples):
        for _ in range(num_samples):
            for state_abbr in EnUsAddressProvider.states_abbr:
                code = faker.postalcode_in_state(state_abbr)
                assert re.fullmatch(r"\d{5}", code)
                assert int(code) >= EnUsAddressProvider.states_postcode[state_abbr][0]
                assert int(code) <= EnUsAddressProvider.states_postcode[state_abbr][1]

        with pytest.raises(Exception):
            faker.postalcode_in_state("XX")

    def test_state_abbr_determinism(self, faker):
        faker.seed_instance(0)
        first = faker.state_abbr()
        faker.seed_instance(0)
        assert faker.state_abbr() == first


class TestEsCo:
    """Test es_CO address provider methods"""

    def test_department_code(self, faker, num_samples):
        for _ in range(num_samples):
            department_code = faker.department_code()
            assert isinstance(department_code, str)
            assert department_code in EsCoAddressProvider.departments

    def test_department(self, faker, num_samples):
        for _ in range(num_samples):
            department = faker.department()
            assert isinstance(department, str)
            assert department in EsCoAddressProvider.departments.values()

    def test_municipality_code(self, faker, num_samples):
        municipality_codes = {municipality_code for municipality_code, _ in EsCoAddressProvider.municipalities}
        for _ in range(num_samples):
            municipality_code = faker.municipality_code()
            assert isinstance(municipality_code, str)
            assert municipality_code in municipality_codes

    def test_municipality(self, faker, num_samples):
        municipalities = {municipality for _, municipality in EsCoAddressProvider.municipalities}
        for _ in range(num_samples):
            municipality = faker.municipality()
            city = faker.city()
            assert isinstance(municipality, str)
            assert isinstance(city, str)
            assert municipality in municipalities
            assert city in municipalities

    def test_street_prefix(self, faker, num_samples):
        for _ in range(num_samples):
            street_prefix = faker.street_prefix()
            assert isinstance(street_prefix, str)
            assert street_prefix in EsCoAddressProvider.street_prefixes

    def test_street_suffix(self, faker, num_samples):
        for _ in range(num_samples):
            street_suffix = faker.street_suffix()
            assert isinstance(street_suffix, str)

    def test_street_name(self, faker, num_samples):
        for _ in range(num_samples):
            street_name = faker.street_name()
            assert isinstance(street_name, str)

    def test_building_number(self, faker, num_samples):
        for _ in range(num_samples):
            building_number = faker.building_number()
            assert isinstance(building_number, str)
            assert re.fullmatch(r"\d{1,2}[A-Z]?-\d{1,2}", building_number)

    def test_secondary_address(self, faker, num_samples):
        for _ in range(num_samples):
            secondary_address = faker.secondary_address()
            assert isinstance(secondary_address, str)

    def test_street_address(self, faker, num_samples):
        for _ in range(num_samples):
            street_address = faker.street_address()
            assert isinstance(street_address, str)

    def test_postcode(self, faker, num_samples):
        for _ in range(num_samples):
            postcode = faker.postcode()
            assert isinstance(postcode, str)
            assert re.fullmatch(r"\d{6}", postcode)

    def test_address(self, faker, num_samples):
        for _ in range(num_samples):
            address = faker.address()
            assert isinstance(address, str)


class TestEsEs:
    """Test es_ES address provider methods"""

    def test_state_name(self, faker, num_samples):
        for _ in range(num_samples):
            state_name = faker.state_name()
            assert isinstance(state_name, str)
            assert state_name in EsEsAddressProvider.states

    def test_street_prefix(self, faker, num_samples):
        for _ in range(num_samples):
            street_prefix = faker.street_prefix()
            assert isinstance(street_prefix, str)
            assert street_prefix in EsEsAddressProvider.street_prefixes

    def test_secondary_address(self, faker, num_samples):
        for _ in range(num_samples):
            secondary_address = faker.secondary_address()
            assert isinstance(secondary_address, str)
            assert re.fullmatch(r"Apt\. \d{2}|Piso \d|Puerta \d", secondary_address)

    def test_regions(self, faker, num_samples):
        for _ in range(num_samples):
            region = faker.region()
            assert isinstance(region, str)
            assert region in EsEsAddressProvider.regions

    def test_autonomous_community(self, faker, num_samples):
        for _ in range(num_samples):
            # Spanish regions, also known as "autonomous communities"
            autonomous_community = faker.autonomous_community()
            assert isinstance(autonomous_community, str)
            assert autonomous_community in EsEsAddressProvider.regions

    def test_postcode(self, faker, num_samples):
        for _ in range(num_samples):
            postcode = faker.postcode()
            assert isinstance(postcode, str)
            assert len(postcode) == 5
            assert 1000 <= int(postcode) <= 52100


class TestEsMx:
    """Test es_MX address provider methods"""

    def test_city_prefix(self, faker, num_samples):
        for _ in range(num_samples):
            city_prefix = faker.city_prefix()
            assert isinstance(city_prefix, str)
            assert city_prefix in EsMxAddressProvider.city_prefixes

    def test_city_suffix(self, faker, num_samples):
        for _ in range(num_samples):
            city_suffix = faker.city_suffix()
            assert isinstance(city_suffix, str)
            assert city_suffix in EsMxAddressProvider.city_suffixes

    def test_city_adjective(self, faker, num_samples):
        for _ in range(num_samples):
            city_adjective = faker.city_adjective()
            assert isinstance(city_adjective, str)
            assert city_adjective in EsMxAddressProvider.city_adjectives

    def test_street_prefix(self, faker, num_samples):
        for _ in range(num_samples):
            street_prefix = faker.street_prefix()
            assert isinstance(street_prefix, str)
            assert street_prefix in EsMxAddressProvider.street_prefixes

    def test_secondary_address(self, faker, num_samples):
        for _ in range(num_samples):
            secondary_address = faker.secondary_address()
            assert isinstance(secondary_address, str)
            assert re.fullmatch(
                r"\d{3} \d{3}|\d{3} Interior \d{3}|\d{3} Edif\. \d{3} , Depto\. \d{3}",
                secondary_address,
            )

    def test_state(self, faker, num_samples):
        states = [state_name for state_abbr, state_name in EsMxAddressProvider.states]
        for _ in range(num_samples):
            state = faker.state()
            assert isinstance(state, str)
            assert state in states

    def test_state_abbr(self, faker, num_samples):
        state_abbrs = [state_abbr for state_abbr, state_name in EsMxAddressProvider.states]
        for _ in range(num_samples):
            state_abbr = faker.state_abbr()
            assert isinstance(state_abbr, str)
            assert state_abbr in state_abbrs


class TestFaIr:
    """Test fa_IR address provider methods"""

    def test_city_prefix(self, faker, num_samples):
        for _ in range(num_samples):
            city_prefix = faker.city_prefix()
            assert isinstance(city_prefix, str)
            assert city_prefix in FaIrAddressProvider.city_prefixes

    def test_secondary_address(self, faker, num_samples):
        for _ in range(num_samples):
            secondary_address = faker.secondary_address()
            assert isinstance(secondary_address, str)
            assert re.fullmatch(r"(?:سوئیت|واحد) \d{3}", secondary_address)

    def test_state(self, faker, num_samples):
        for _ in range(num_samples):
            state = faker.state()
            assert isinstance(state, str)
            assert state in FaIrAddressProvider.states


class TestFrFr:
    """Test fr_FR address provider methods"""

    def test_street_prefix(self, faker, num_samples):
        for _ in range(num_samples):
            street_prefix = faker.street_prefix()
            assert isinstance(street_prefix, str)
            assert street_prefix in FrFrAddressProvider.street_prefixes

    def test_city_prefix(self, faker, num_samples):
        for _ in range(num_samples):
            city_prefix = faker.city_prefix()
            assert isinstance(city_prefix, str)
            assert city_prefix in FrFrAddressProvider.city_prefixes

    def test_region(self, faker, num_samples):
        for _ in range(num_samples):
            region = faker.region()
            assert isinstance(region, str)
            assert region in FrFrAddressProvider.regions

    def test_department(self, faker, num_samples):
        for _ in range(num_samples):
            department = faker.department()
            assert isinstance(department, tuple)
            assert department in FrFrAddressProvider.departments

    def test_department_name(self, faker, num_samples):
        department_names = [dept_name for dept_num, dept_name in FrFrAddressProvider.departments]
        for _ in range(num_samples):
            department_name = faker.department_name()
            assert isinstance(department_name, str)
            assert department_name in department_names

    def test_department_number(self, faker, num_samples):
        department_numbers = [dept_num for dept_num, dept_name in FrFrAddressProvider.departments]
        for _ in range(num_samples):
            department_number = faker.department_number()
            assert isinstance(department_number, str)
            assert department_number in department_numbers

    def test_postcode(self, faker, num_samples):
        department_numbers = [dept_num for dept_num, dept_name in FrFrAddressProvider.departments]
        for _ in range(num_samples):
            postcode = faker.postcode()
            assert isinstance(postcode, str)
            assert len(postcode) == 5
            assert (
                postcode[:3] in department_numbers  # for 3 digits departments number
                or postcode[:2] == "20"  # for Corsica : "2A" or "2B"
                or postcode[:2] in department_numbers  # any other
            )


class TestHeIl:
    """Test he_IL address provider methods"""

    def test_city_name(self, faker, num_samples):
        for _ in range(num_samples):
            city_name = faker.city_name()
            assert isinstance(city_name, str)
            assert city_name in HeIlAddressProvider.city_names

    def test_street_title(self, faker, num_samples):
        for _ in range(num_samples):
            street_title = faker.street_title()
            assert isinstance(street_title, str)
            assert street_title in HeIlAddressProvider.street_titles


class TestHiIn:
    """Test hi_IN address provider methods"""

    def test_city_name(self, faker, num_samples):
        for _ in range(num_samples):
            city_name = faker.city_name()
            assert isinstance(city_name, str)
            assert city_name in HiInAddressProvider.cities

    def test_state(self, faker, num_samples):
        for _ in range(num_samples):
            state = faker.state()
            assert isinstance(state, str)
            assert state in HiInAddressProvider.states


class TestTaIn:
    """Test ta_IN address provider methods"""

    def test_city_name(self, faker, num_samples):
        for _ in range(num_samples):
            city_name = faker.city_name()
            assert isinstance(city_name, str)
            assert city_name in TaInAddressProvider.cities

    def test_state(self, faker, num_samples):
        for _ in range(num_samples):
            state = faker.state()
            assert isinstance(state, str)
            assert state in TaInAddressProvider.states


class TestFiFi:
    """Test fi_FI address provider methods"""

    def test_city(self, faker, num_samples):
        for _ in range(num_samples):
            city = faker.city()
            assert isinstance(city, str)
            assert city in FiFiAddressProvider.cities

    def test_street_suffix(self, faker, num_samples):
        for _ in range(num_samples):
            suffix = faker.street_suffix()
            assert isinstance(suffix, str)
            assert suffix in FiFiAddressProvider.street_suffixes

    def test_state(self, faker, num_samples):
        for _ in range(num_samples):
            state = faker.state()
            assert isinstance(state, str)
            assert state in FiFiAddressProvider.states


class TestHrHr:
    """Test hr_HR address provider methods"""

    def test_city_name(self, faker, num_samples):
        for _ in range(num_samples):
            city_name = faker.city_name()
            assert isinstance(city_name, str)
            assert city_name in HrHrAddressProvider.cities

    def test_street_name(self, faker, num_samples):
        for _ in range(num_samples):
            street_name = faker.street_name()
            assert isinstance(street_name, str)
            assert street_name in HrHrAddressProvider.streets

    def test_state(self, faker, num_samples):
        for _ in range(num_samples):
            state = faker.state()
            assert isinstance(state, str)
            assert state in HrHrAddressProvider.states


class TestHyAm:
    """Test hy_AM address provider methods"""

    def test_address(self, faker, num_samples):
        for _ in range(num_samples):
            address = faker.address()
            assert isinstance(address, str)

    def test_building_number(self, faker, num_samples):
        for _ in range(num_samples):
            building_number = faker.building_number()
            assert isinstance(building_number, str)
            assert 0 <= int(building_number) <= 999

    def test_city(self, faker, num_samples):
        for _ in range(num_samples):
            city = faker.city()
            assert isinstance(city, str)
            assert city in HyAmAddressProvider.cities

    def test_city_prefix(self, faker, num_samples):
        for _ in range(num_samples):
            city_prefix = faker.city_prefix()
            assert isinstance(city_prefix, str)
            assert city_prefix in HyAmAddressProvider.city_prefixes

    def test_country(self, faker, num_samples):
        for _ in range(num_samples):
            country = faker.country()
            assert isinstance(country, str)
            assert country in HyAmAddressProvider.countries

    def test_postcode(self, faker, num_samples):
        for _ in range(num_samples):
            postcode = faker.postcode()
            assert isinstance(postcode, str)
            assert 200 <= int(postcode) <= 4299

    def test_postcode_in_state(self, faker, num_samples):
        for _ in range(num_samples):
            for state_abbr in HyAmAddressProvider.states_abbr:
                code = faker.postcode_in_state(state_abbr)
                assert re.fullmatch(r"\d{4}", code)
                assert int(code) >= HyAmAddressProvider.states_postcode[state_abbr][0]
                assert int(code) <= HyAmAddressProvider.states_postcode[state_abbr][1]

        with pytest.raises(Exception):
            faker.postcode_in_state("XX")

    def test_secondary_address(self, faker, num_samples):
        for _ in range(num_samples):
            secondary_address = faker.secondary_address()
            assert isinstance(secondary_address, str)
            assert re.fullmatch(r"բն\. \d{1,2}", secondary_address)

    def test_state(self, faker, num_samples):
        for _ in range(num_samples):
            state = faker.state()
            assert isinstance(state, str)
            assert state in HyAmAddressProvider.states

    def test_state_abbr(self, faker, num_samples):
        for _ in range(num_samples):
            state_abbr = faker.state_abbr()
            assert isinstance(state_abbr, str)
            assert state_abbr in HyAmAddressProvider.states_abbr
            assert state_abbr.isupper()

    def test_street(self, faker, num_samples):
        for _ in range(num_samples):
            street = faker.street()
            assert isinstance(street, str)
            assert street in HyAmAddressProvider.streets

    def test_street_address(self, faker, num_samples):
        for _ in range(num_samples):
            street_address = faker.street_address()
            assert isinstance(street_address, str)

    def test_street_name(self, faker, num_samples):
        for _ in range(num_samples):
            street_name = faker.street_name()
            assert isinstance(street_name, str)

    def test_street_prefix(self, faker, num_samples):
        for _ in range(num_samples):
            street_prefix = faker.street_prefix()
            assert isinstance(street_prefix, str)
            assert street_prefix in HyAmAddressProvider.street_prefixes

    def test_street_suffix(self, faker, num_samples):
        for _ in range(num_samples):
            suffix = faker.street_suffix()
            assert isinstance(suffix, str)
            assert suffix in HyAmAddressProvider.street_suffixes

    def test_village(self, faker, num_samples):
        for _ in range(num_samples):
            village = faker.village()
            assert isinstance(village, str)
            assert village in HyAmAddressProvider.villages

    def test_village_prefix(self, faker, num_samples):
        for _ in range(num_samples):
            village_prefix = faker.village_prefix()
            assert isinstance(village_prefix, str)
            assert village_prefix in HyAmAddressProvider.village_prefixes


class TestItIt:
    """Test it_IT address provider methods"""

    def test_city(self, faker, num_samples):
        for _ in range(num_samples):
            city = faker.city()
            assert isinstance(city, str)
            assert city in ItItAddressProvider.cities

    def test_postcode_city_province(self, faker, num_samples):
        for _ in range(num_samples):
            postcode_city_province = faker.postcode_city_province()
            assert isinstance(postcode_city_province, str)
            match = re.fullmatch(r"(?P<cap>\d{5}), (?P<city>.*) \((?P<province>[A-Z]{2})\)", postcode_city_province)
            assert match
            assert match.group("cap") in ItItAddressProvider.postcode_formats
            assert match.group("city") in ItItAddressProvider.cities
            assert match.group("province") in ItItAddressProvider.states_abbr

    def test_city_prefix(self, faker, num_samples):
        for _ in range(num_samples):
            city_prefix = faker.city_prefix()
            assert isinstance(city_prefix, str)
            assert city_prefix in ItItAddressProvider.city_prefixes

    def test_secondary_address(self, faker, num_samples):
        for _ in range(num_samples):
            secondary_address = faker.secondary_address()
            assert isinstance(secondary_address, str)
            match = re.findall(r".* \d{1,2}", secondary_address)
            assert match

    def test_administrative_unit(self, faker, num_samples):
        for _ in range(num_samples):
            administrative_unit = faker.administrative_unit()
            assert isinstance(administrative_unit, str)
            assert administrative_unit in ItItAddressProvider.states

    def test_state_abbr(self, faker, num_samples):
        for _ in range(num_samples):
            state_abbr = faker.state_abbr()
            assert isinstance(state_abbr, str)
            assert state_abbr in ItItAddressProvider.states_abbr


class TestJaJp:
    """Test ja_JP address provider methods"""

    def test_chome(self, faker, num_samples):
        for _ in range(num_samples):
            chome = faker.chome()
            assert isinstance(chome, str)
            match = re.fullmatch(r"(?P<chome_number>\d{1,2})丁目", chome)
            assert match
            assert 1 <= int(match.group("chome_number")) <= 42

    def test_ban(self, faker, num_samples):
        for _ in range(num_samples):
            ban = faker.ban()
            assert isinstance(ban, str)
            match = re.fullmatch(r"(?P<ban_number>\d{1,2})番", ban)
            assert match
            assert 1 <= int(match.group("ban_number")) <= 27

    def test_gou(self, faker, num_samples):
        for _ in range(num_samples):
            gou = faker.gou()
            assert isinstance(gou, str)
            match = re.fullmatch(r"(?P<gou_number>\d{1,2})号", gou)
            assert match
            assert 1 <= int(match.group("gou_number")) <= 20

    def test_town(self, faker, num_samples):
        for _ in range(num_samples):
            town = faker.town()
            assert isinstance(town, str)
            assert town in JaJpAddressProvider.towns

    def test_prefecture(self, faker, num_samples):
        for _ in range(num_samples):
            prefecture = faker.prefecture()
            assert isinstance(prefecture, str)
            assert prefecture in JaJpAddressProvider.prefectures

    def test_city(self, faker, num_samples):
        for _ in range(num_samples):
            city = faker.city()
            assert isinstance(city, str)
            assert city in JaJpAddressProvider.cities

    def test_country(self, faker, num_samples):
        for _ in range(num_samples):
            country = faker.country()
            assert isinstance(country, str)
            assert country in JaJpAddressProvider.countries

    def test_building_name(self, faker, num_samples):
        for _ in range(num_samples):
            building_name = faker.building_name()
            assert isinstance(building_name, str)
            assert building_name in JaJpAddressProvider.building_names

    def test_address(self, faker, num_samples):
        for _ in range(num_samples):
            address = faker.address()
            assert isinstance(address, str)

    def test_postcode(self, faker, num_samples):
        for _ in range(num_samples):
            postcode = faker.postcode()
            assert isinstance(postcode, str)
            assert re.fullmatch(r"\d{3}-\d{4}", postcode)

    def test_zipcode(self, faker, num_samples):
        for _ in range(num_samples):
            zipcode = faker.zipcode()
            assert isinstance(zipcode, str)
            assert re.fullmatch(r"\d{3}-\d{4}", zipcode)


class TestKoKr:
    """Test ko_KR address provider methods"""

    def test_old_postal_code(self, faker, num_samples):
        for _ in range(num_samples):
            old_postal_code = faker.old_postal_code()
            assert isinstance(old_postal_code, str)
            assert re.fullmatch(r"\d{3}-\d{3}", old_postal_code)

    def test_postal_code(self, faker, num_samples):
        for _ in range(num_samples):
            postal_code = faker.postal_code()
            assert isinstance(postal_code, str)
            assert re.fullmatch(r"\d{5}", postal_code)

    def test_postcode(self, faker, num_samples):
        for _ in range(num_samples):
            postcode = faker.postcode()
            assert isinstance(postcode, str)
            assert re.fullmatch(r"\d{5}", postcode)

    def test_city(self, faker, num_samples):
        for _ in range(num_samples):
            city = faker.city()
            assert isinstance(city, str)

    def test_borough(self, faker, num_samples):
        for _ in range(num_samples):
            borough = faker.borough()
            assert isinstance(borough, str)
            assert borough in KoKrAddressProvider.boroughs

    def test_town(self, faker, num_samples):
        for _ in range(num_samples):
            town = faker.town()
            assert isinstance(town, str)

    def test_town_suffix(self, faker, num_samples):
        for _ in range(num_samples):
            town_suffix = faker.town_suffix()
            assert isinstance(town_suffix, str)
            assert town_suffix in KoKrAddressProvider.town_suffixes

    def test_building_name(self, faker, num_samples):
        for _ in range(num_samples):
            building_name = faker.building_name()
            assert isinstance(building_name, str)

    def test_building_suffix(self, faker, num_samples):
        for _ in range(num_samples):
            building_suffix = faker.building_suffix()
            assert isinstance(building_suffix, str)
            assert building_suffix in KoKrAddressProvider.building_suffixes

    def test_building_dong(self, faker, num_samples):
        for _ in range(num_samples):
            building_dong = faker.building_dong()
            assert isinstance(building_dong, str)


class TestNeNp:
    """Test ne_NP address provider methods"""

    def test_province(self, faker, num_samples):
        for _ in range(num_samples):
            province = faker.province()
            assert isinstance(province, str)
            assert province in NeNpAddressProvider.provinces

    def test_district(self, faker, num_samples):
        for _ in range(num_samples):
            district = faker.district()
            assert isinstance(district, str)
            assert district in NeNpAddressProvider.districts

    def test_city(self, faker, num_samples):
        for _ in range(num_samples):
            city = faker.city()
            assert isinstance(city, str)
            assert city in NeNpAddressProvider.cities

    def test_country(self, faker, num_samples):
        for _ in range(num_samples):
            country = faker.country()
            assert isinstance(country, str)
            assert country in NeNpAddressProvider.countries


class TestNoNo:
    """Test no_NO address provider methods"""

    def test_postcode(self, faker):
        for _ in range(100):
            assert re.fullmatch(r"^[0-9]{4}$", faker.postcode())

    def test_city_suffix(self, faker, num_samples):
        for _ in range(num_samples):
            city_suffix = faker.city_suffix()
            assert isinstance(city_suffix, str)
            assert city_suffix in NoNoAddressProvider.city_suffixes

    def test_street_suffix(self, faker, num_samples):
        for _ in range(num_samples):
            street_suffix = faker.street_suffix()
            assert isinstance(street_suffix, str)
            assert street_suffix in NoNoAddressProvider.street_suffixes

    def test_address(self, faker, num_samples):
        for _ in range(num_samples):
            address = faker.address()
            assert isinstance(address, str)


class TestZhTw:
    """Test zh_TW address provider methods"""

    def test_postcode(self, faker, num_samples):
        for _ in range(num_samples):
            postcode = faker.postcode()
            assert isinstance(postcode, str)
            assert re.fullmatch(r"[1-9]\d{2}(?:\d{2})?", postcode)

    def test_city_name(self, faker, num_samples):
        for _ in range(num_samples):
            city_name = faker.city_name()
            assert isinstance(city_name, str)
            assert city_name in ZhTwAddressProvider.cities

    def test_city_suffix(self, faker, num_samples):
        for _ in range(num_samples):
            city_suffix = faker.city_suffix()
            assert isinstance(city_suffix, str)
            assert city_suffix in ZhTwAddressProvider.city_suffixes

    def test_city(self, faker, num_samples):
        city_pattern: Pattern = re.compile(r"(?P<city_name>.*?)[市縣]?")
        for _ in range(num_samples):
            city = faker.city()
            assert isinstance(city, str)
            match = city_pattern.fullmatch(city)
            assert match
            assert match.group("city_name") in ZhTwAddressProvider.cities

    def test_country(self, faker, num_samples):
        for _ in range(num_samples):
            country = faker.country()
            assert isinstance(country, str)
            assert country in ZhTwAddressProvider.countries

    def test_street_name(self, faker, num_samples):
        for _ in range(num_samples):
            street_name = faker.street_name()
            assert isinstance(street_name, str)
            assert street_name in ZhTwAddressProvider.street_names

    def test_address(self, faker, num_samples):
        for _ in range(num_samples):
            address = faker.address()
            assert isinstance(address, str)


class TestZhCn:
    """Test zh_CN address provider methods"""

    def test_postcode(self, faker, num_samples):
        for _ in range(num_samples):
            postcode = faker.postcode()
            assert isinstance(postcode, str)
            assert re.fullmatch(r"[1-9]\d{5}", postcode)

    def test_city_name(self, faker, num_samples):
        for _ in range(num_samples):
            city_name = faker.city_name()
            assert isinstance(city_name, str)
            assert city_name in ZhCnAddressProvider.cities

    def test_city_suffix(self, faker, num_samples):
        for _ in range(num_samples):
            city_suffix = faker.city_suffix()
            assert isinstance(city_suffix, str)
            assert city_suffix in ZhCnAddressProvider.city_suffixes

    def test_city(self, faker, num_samples):
        city_pattern: Pattern = re.compile(r".*?[市县]")
        for _ in range(num_samples):
            city = faker.city()
            assert isinstance(city, str)
            assert city_pattern.fullmatch(city)

    def test_province(self, faker, num_samples):
        for _ in range(num_samples):
            province = faker.province()
            assert isinstance(province, str)
            assert province in ZhCnAddressProvider.provinces

    def test_district(self, faker, num_samples):
        for _ in range(num_samples):
            district = faker.district()
            assert isinstance(district, str)
            assert district in ZhCnAddressProvider.districts

    def test_country(self, faker, num_samples):
        for _ in range(num_samples):
            country = faker.country()
            assert isinstance(country, str)
            assert country in ZhCnAddressProvider.countries

    def test_street_name(self, faker, num_samples):
        for _ in range(num_samples):
            street_name = faker.street_name()
            assert isinstance(street_name, str)

    def test_address(self, faker, num_samples):
        for _ in range(num_samples):
            address = faker.address()
            assert isinstance(address, str)


class TestPtBr:
    """Test pt_BR address provider methods"""

    def test_country(self, faker, num_samples):
        for _ in range(num_samples):
            country = faker.country()
            assert isinstance(country, str)
            assert country in PtBrAddressProvider.countries

    def test_bairro(self, faker, num_samples):
        for _ in range(num_samples):
            bairro = faker.bairro()
            assert isinstance(bairro, str)
            assert bairro in PtBrAddressProvider.bairros

    def test_neighborhood(self, faker, num_samples):
        for _ in range(num_samples):
            neighborhood = faker.neighborhood()
            assert isinstance(neighborhood, str)
            assert neighborhood in PtBrAddressProvider.bairros

    def test_estado(self, faker, num_samples):
        for _ in range(num_samples):
            estado = faker.estado()
            assert isinstance(estado, tuple)
            assert estado in PtBrAddressProvider.estados

    def test_estado_nome(self, faker, num_samples):
        state_names = [state_name for state_abbr, state_name in PtBrAddressProvider.estados]
        for _ in range(num_samples):
            estado_nome = faker.estado_nome()
            assert isinstance(estado_nome, str)
            assert estado_nome in state_names

    def test_estado_sigla(self, faker, num_samples):
        state_abbrs = [state_abbr for state_abbr, state_name in PtBrAddressProvider.estados]
        for _ in range(num_samples):
            estado_sigla = faker.estado_sigla()
            assert isinstance(estado_sigla, str)
            assert estado_sigla in state_abbrs

    def test_address(self, faker, num_samples):
        for _ in range(num_samples):
            street = faker.street_name()
            assert isinstance(street, str)
            city = faker.street_address()
            assert isinstance(city, str)
            address = faker.address()
            assert isinstance(address, str)

    def test_raw_postcode(self, faker, num_samples):
        for _ in range(num_samples):
            postcode = faker.postcode(formatted=False)
            assert isinstance(postcode, str)
            assert re.fullmatch(r"\d{8}", postcode)

    def test_formatted_postcode(self, faker, num_samples):
        for _ in range(num_samples):
            postcode = faker.postcode()
            assert isinstance(postcode, str)
            assert re.fullmatch(r"\d{5}-?\d{3}", postcode)


class TestPtPt:
    """Test pt_PT address provider methods"""

    def test_distrito(self, faker, num_samples):
        for _ in range(num_samples):
            distrito = faker.distrito()
            assert isinstance(distrito, str)
            assert distrito in PtPtAddressProvider.distritos

    def test_concelho(self, faker, num_samples):
        for _ in range(num_samples):
            concelho = faker.concelho()
            assert isinstance(concelho, str)
            assert concelho in PtPtAddressProvider.concelhos

    def test_freguesia(self, faker, num_samples):
        for _ in range(num_samples):
            freguesia = faker.freguesia()
            assert isinstance(freguesia, str)
            assert freguesia in PtPtAddressProvider.freguesias

    def test_place_name(self, faker, num_samples):
        for _ in range(num_samples):
            place_name = faker.place_name()
            assert isinstance(place_name, str)
            assert place_name in PtPtAddressProvider.places


class TestEnPh:
    """Test en_PH address provider methods"""

    @classmethod
    def setup_class(cls):
        cls.building_number_pattern: Pattern = re.compile(
            r"(?:[1-9]|[1-9]\d{1,3})(?:[A-J]|\s[A-J]|-[A-J]|\sUnit\s[A-J])?",
        )
        cls.address_pattern: Pattern = re.compile(
            r"(?P<street_address>.*), (?P<lgu>.*?), (?P<postcode>\d{4}) (?P<province>.*?)",
        )
        cls.metro_manila_postcodes = EnPhAddressProvider.metro_manila_postcodes
        cls.luzon_province_postcodes = EnPhAddressProvider.luzon_province_postcodes
        cls.visayas_province_postcodes = EnPhAddressProvider.visayas_province_postcodes
        cls.mindanao_province_postcodes = EnPhAddressProvider.mindanao_province_postcodes
        cls.postcodes = EnPhAddressProvider.postcodes
        cls.provinces = EnPhAddressProvider.provinces
        cls.province_lgus = EnPhAddressProvider.province_lgus
        cls.metro_manila_lgus = EnPhAddressProvider.metro_manila_lgus

    def test_metro_manila_postcode(self, faker, num_samples):
        for _ in range(num_samples):
            assert int(faker.metro_manila_postcode()) in self.metro_manila_postcodes

    def test_luzon_province_postcode(self, faker, num_samples):
        for _ in range(num_samples):
            assert int(faker.luzon_province_postcode()) in self.luzon_province_postcodes

    def test_visayas_province_postcode(self, faker, num_samples):
        for _ in range(num_samples):
            assert int(faker.visayas_province_postcode()) in self.visayas_province_postcodes

    def test_mindanao_province_postcode(self, faker, num_samples):
        for _ in range(num_samples):
            assert int(faker.mindanao_province_postcode()) in self.mindanao_province_postcodes

    def test_postcode(self, faker, num_samples):
        for _ in range(num_samples):
            assert int(faker.postcode()) in self.postcodes

    def test_building_number(self, faker, num_samples):
        for _ in range(num_samples):
            assert self.building_number_pattern.fullmatch(faker.building_number())

    def test_floor_unit_number(self, faker, num_samples):
        for _ in range(num_samples):
            number = faker.floor_unit_number()
            assert 2 <= int(number[:-2]) <= 99
            assert 1 <= int(number[-2:]) <= 40

    def test_ordinal_floor_number(self, faker, num_samples):
        for _ in range(num_samples):
            floor_number = faker.ordinal_floor_number()
            assert floor_number[-2:] in ["th", "st", "nd", "rd"]

    def test_address(self, faker, num_samples):
        for _ in range(num_samples):
            address = faker.address()
            match = self.address_pattern.fullmatch(address)
            street_address = match.group("street_address")
            lgu = match.group("lgu")
            postcode = match.group("postcode")
            province = match.group("province")
            assert match
            assert street_address
            assert lgu in self.province_lgus or lgu in self.metro_manila_lgus
            assert int(postcode) in self.postcodes
            assert province in self.provinces or province == "Metro Manila"


class TestFilPh(TestEnPh):
    """Test fil_PH address provider methods"""

    @classmethod
    def setup_class(cls):
        cls.building_number_pattern: Pattern = re.compile(
            r"(?:[1-9]|[1-9]\d{1,3})(?:[A-J]|\s[A-J]|-[A-J]|\sUnit\s[A-J])?",
        )
        cls.address_pattern: Pattern = re.compile(
            r"(?P<street_address>.*), (?P<lgu>.*?), (?P<postcode>\d{4}) (?P<province>.*?)",
        )
        cls.metro_manila_postcodes = EnPhAddressProvider.metro_manila_postcodes
        cls.luzon_province_postcodes = EnPhAddressProvider.luzon_province_postcodes
        cls.visayas_province_postcodes = EnPhAddressProvider.visayas_province_postcodes
        cls.mindanao_province_postcodes = EnPhAddressProvider.mindanao_province_postcodes
        cls.postcodes = EnPhAddressProvider.postcodes
        cls.provinces = EnPhAddressProvider.provinces
        cls.province_lgus = EnPhAddressProvider.province_lgus
        cls.metro_manila_lgus = EnPhAddressProvider.metro_manila_lgus

    def test_metro_manila_postcode(self, faker, num_samples):
        for _ in range(num_samples):
            assert int(faker.metro_manila_postcode()) in self.metro_manila_postcodes

    def test_luzon_province_postcode(self, faker, num_samples):
        for _ in range(num_samples):
            assert int(faker.luzon_province_postcode()) in self.luzon_province_postcodes

    def test_visayas_province_postcode(self, faker, num_samples):
        for _ in range(num_samples):
            assert int(faker.visayas_province_postcode()) in self.visayas_province_postcodes

    def test_mindanao_province_postcode(self, faker, num_samples):
        for _ in range(num_samples):
            assert int(faker.mindanao_province_postcode()) in self.mindanao_province_postcodes

    def test_postcode(self, faker, num_samples):
        for _ in range(num_samples):
            assert int(faker.postcode()) in self.postcodes

    def test_building_number(self, faker, num_samples):
        for _ in range(num_samples):
            assert self.building_number_pattern.fullmatch(faker.building_number())

    def test_floor_unit_number(self, faker, num_samples):
        for _ in range(num_samples):
            number = faker.floor_unit_number()
            assert 2 <= int(number[:-2]) <= 99
            assert 1 <= int(number[-2:]) <= 40

    def test_ordinal_floor_number(self, faker, num_samples):
        for _ in range(num_samples):
            floor_number = faker.ordinal_floor_number()
            assert floor_number[-2:] in ["th", "st", "nd", "rd"]

    def test_address(self, faker, num_samples):
        for _ in range(num_samples):
            address = faker.address()
            match = self.address_pattern.fullmatch(address)
            street_address = match.group("street_address")
            lgu = match.group("lgu")
            postcode = match.group("postcode")
            province = match.group("province")
            assert match
            assert street_address
            assert lgu in self.province_lgus or lgu in self.metro_manila_lgus
            assert int(postcode) in self.postcodes
            assert province in self.provinces or province == "Metro Manila"


class TestTlPh(TestEnPh):
    """Test tl_PH address provider methods"""

    pass


class TestRuRu:
    """Test ru_RU address provider methods"""

    def test_city_name(self, faker, num_samples):
        for _ in range(num_samples):
            city = faker.city_name()
            assert isinstance(city, str)
            assert city in RuRuAddressProvider.city_names

    def test_country(self, faker, num_samples):
        for _ in range(num_samples):
            country = faker.country()
            assert isinstance(country, str)
            assert country in RuRuAddressProvider.countries

    def test_region(self, faker, num_samples):
        region_pattern: Pattern = re.compile(
            r"(?:респ\. (?P<region_republic>.*))|"
            r"(?:(?P<region_krai>.*?) край)|"
            r"(?:(?P<region_oblast>.*?) обл.)|"
            r"(?:(?P<region_ao>.*?) АО)",
        )
        for _ in range(num_samples):
            region = faker.region()
            assert isinstance(region, str)
            match = region_pattern.fullmatch(region)
            assert match
            groupdict = match.groupdict()
            assert any(
                [
                    groupdict.get("region_republic") in RuRuAddressProvider.region_republics,
                    groupdict.get("region_krai") in RuRuAddressProvider.region_krai,
                    groupdict.get("region_oblast") in RuRuAddressProvider.region_oblast,
                    groupdict.get("region_ao") in RuRuAddressProvider.region_ao,
                ]
            )

    def test_postcode(self, faker, num_samples):
        for _ in range(num_samples):
            postcode = faker.postcode()
            assert isinstance(postcode, str)
            assert re.fullmatch(r"\d{6}", postcode)

    def test_city_prefix(self, faker, num_samples):
        for _ in range(num_samples):
            city_prefix = faker.city_prefix()
            assert isinstance(city_prefix, str)
            assert city_prefix in RuRuAddressProvider.city_prefixes

    def test_street_suffix(self, faker, num_samples):
        for _ in range(num_samples):
            street_suffix = faker.street_suffix()
            assert isinstance(street_suffix, str)
            assert street_suffix in RuRuAddressProvider.street_suffixes

    def test_street_title(self, faker, num_samples):
        for _ in range(num_samples):
            street_title = faker.street_title()
            assert isinstance(street_title, str)

    def test_street_name(self, faker, num_samples):
        for _ in range(num_samples):
            street_name = faker.street_name()
            assert isinstance(street_name, str)

    @pytest.mark.parametrize(
        "street_title,street_suffix,expected",
        [
            ("Фрунзе", "ул.", "ул. Фрунзе"),
            ("Ставропольская", "ул.", "ул. Ставропольская"),
            ("Фрунзе", "пр.", "пр. Фрунзе"),
            ("Осенняя", "пр.", "пр. Осенний"),
            ("Гвардейская", "пр.", "пр. Гвардейский"),
            ("Рыбацкая", "пр.", "пр. Рыбацкий"),
            ("Безымянная", "пр.", "пр. Безымянный"),
            ("Проезжая", "ш.", "ш. Проезжее"),
            ("Магистральная", "ш.", "ш. Магистральное"),
        ],
        ids=[
            "feminine_suffix_and_noflex_title",
            "feminine_suffix_and_flex_title",
            "non_feminine_suffix_and_noflex_title",
            "masc_suffix_and_irregular_masc_title",
            "masc_suffix_and_ck_street_stem",
            "masc_suffix_and_uk_street_stem",
            "masc_suffix_and_other_stem",
            "neu_suffx_and_iregular_neu_street_title",
            "neu_suffix_and_regular_street_title",
        ],
    )
    def test_street_name_lexical(self, faker, street_title, street_suffix, expected):
        """Test that random street names are formed correctly, given
        the case of suffixes and streets that have been randomly selected.
        """
        title_patch = mock.patch(
            "faker.providers.address.ru_RU.Provider.street_title",
            autospec=True,
            return_value=street_title,
        )
        suffix_patch = mock.patch(
            "faker.providers.address.ru_RU.Provider.street_suffix",
            autospec=True,
            return_value=street_suffix,
        )

        with title_patch, suffix_patch:
            result = faker.street_name()
            assert result == expected


class TestThTh:
    """Test th_TH address provider methods"""

    def test_country(self, faker, num_samples):
        for _ in range(num_samples):
            country = faker.country()
            assert isinstance(country, str)
            assert country in ThThAddressProvider.countries

    def test_city_name(self, faker, num_samples):
        for _ in range(num_samples):
            city = faker.city_name()
            assert isinstance(city, str)
            assert city in ThThAddressProvider.cities

    def test_province(self, faker, num_samples):
        for _ in range(num_samples):
            province = faker.province()
            assert isinstance(province, str)
            assert province in ThThAddressProvider.provinces

    def test_amphoe(self, faker, num_samples):
        for _ in range(num_samples):
            amphoe = faker.amphoe()
            assert isinstance(amphoe, str)
            assert amphoe in ThThAddressProvider.amphoes

    def test_tambon(self, faker, num_samples):
        for _ in range(num_samples):
            tambon = faker.tambon()
            assert isinstance(tambon, str)

    def test_postcode(self, faker, num_samples):
        for _ in range(num_samples):
            postcode = faker.postcode()
            assert isinstance(postcode, str)
            assert re.fullmatch(r"[1-9]\d{4}", postcode)


class TestEnIn:
    """Test en_IN address provider methods"""

    def test_city_name(self, faker, num_samples):
        for _ in range(num_samples):
            city_name = faker.city_name()
            assert isinstance(city_name, str)
            assert city_name in EnInAddressProvider.cities

    def test_state(self, faker, num_samples):
        for _ in range(num_samples):
            state = faker.state()
            assert isinstance(state, str)
            assert state in EnInAddressProvider.states


class TestSkSk:
    """Test sk_SK address provider methods"""

    def test_street_suffix_short(self, faker, num_samples):
        for _ in range(num_samples):
            street_suffix_short = faker.street_suffix_short()
            assert isinstance(street_suffix_short, str)
            assert street_suffix_short in SkSkAddressProvider.street_suffixes_short

    def test_street_suffix_long(self, faker, num_samples):
        for _ in range(num_samples):
            street_suffix_long = faker.street_suffix_long()
            assert isinstance(street_suffix_long, str)
            assert street_suffix_long in SkSkAddressProvider.street_suffixes_long

    def test_city_name(self, faker, num_samples):
        for _ in range(num_samples):
            city = faker.city_name()
            assert isinstance(city, str)
            assert city in SkSkAddressProvider.cities

    def test_street_name(self, faker, num_samples):
        for _ in range(num_samples):
            street_name = faker.street_name()
            assert isinstance(street_name, str)
            assert street_name in SkSkAddressProvider.streets

    def test_state(self, faker, num_samples):
        for _ in range(num_samples):
            state = faker.state()
            assert isinstance(state, str)
            assert state in SkSkAddressProvider.states

    def test_postcode(self, faker, num_samples):
        for _ in range(num_samples):
            postcode = faker.postcode()
            assert isinstance(postcode, str)
            assert re.fullmatch(r"\d{3} \d{2}", postcode)

    def test_city_with_postcode(self, faker, num_samples):
        for _ in range(num_samples):
            city_with_postcode = faker.city_with_postcode()
            assert isinstance(city_with_postcode, str)
            match = re.fullmatch(r"\d{3} \d{2} (?P<city>.*)", city_with_postcode)
            assert match.group("city") in SkSkAddressProvider.cities


class TestDeCh:
    """Test de_CH address provider methods"""

    def test_canton_name(self, faker, num_samples):
        for _ in range(num_samples):
            canton_name = faker.canton_name()
            assert isinstance(canton_name, str)
            assert any(canton_name == cantons[1] for cantons in DeChAddressProvider.cantons)

    def test_canton_code(self, faker, num_samples):
        for _ in range(num_samples):
            canton_code = faker.canton_code()
            assert isinstance(canton_code, str)
            assert any(canton_code == cantons[0] for cantons in DeChAddressProvider.cantons)

    def test_canton(self, faker, num_samples):
        for _ in range(num_samples):
            canton = faker.canton()
            assert isinstance(canton, tuple)
            assert canton in DeChAddressProvider.cantons

    def test_city(self, faker, num_samples):
        for _ in range(num_samples):
            city = faker.city_name()
            assert isinstance(city, str)
            assert city in DeChAddressProvider.cities


class TestRoRo:
    """Test ro_RO address provider methods"""

    def test_address(self, faker, num_samples):
        for _ in range(num_samples):
            address = faker.address()
            assert isinstance(address, str)

    def test_street_address(self, faker, num_samples):
        for _ in range(num_samples):
            street_address = faker.street_address()
            assert isinstance(street_address, str)

    def test_street_name(self, faker, num_samples):
        for _ in range(num_samples):
            street_name = faker.street_name()
            assert isinstance(street_name, str)

    def test_street_prefix(self, faker, num_samples):
        for _ in range(num_samples):
            street_prefix = faker.street_prefix()
            assert isinstance(street_prefix, str)
            assert street_prefix in RoRoAddressProvider.street_prefixes

    def test_building_number(self, faker, num_samples):
        for _ in range(num_samples):
            building_number = faker.building_number()
            assert isinstance(building_number, str)
            assert building_number[:3] == "Nr."

    def test_secondary_address(self, faker, num_samples):
        for _ in range(num_samples):
            secondary_address = faker.secondary_address()
            assert isinstance(secondary_address, str)
            assert re.fullmatch(
                r"Bl. \d{2}  Sc. \d{2} Ap. \d{3}",
                secondary_address,
            )

    def test_city(self, faker, num_samples):
        for _ in range(num_samples):
            city = faker.city()
            assert isinstance(city, str)
            assert city in RoRoAddressProvider.cities

    def test_city_name(self, faker, num_samples):
        for _ in range(num_samples):
            city = faker.city_name()
            assert isinstance(city, str)
            assert city in RoRoAddressProvider.cities

    def test_state(self, faker, num_samples):
        states = [state_name for state_abbr, state_name in RoRoAddressProvider.states]
        for _ in range(num_samples):
            state = faker.state()
            assert isinstance(state, str)
            assert state in states

    def test_state_abbr(self, faker, num_samples):
        state_abbrs = [state_abbr for state_abbr, state_name in RoRoAddressProvider.states]
        for _ in range(num_samples):
            state_abbr = faker.state_abbr()
            assert isinstance(state_abbr, str)
            assert state_abbr in state_abbrs
            assert state_abbr.isupper()

    def test_postcode(self, faker, num_samples):
        for _ in range(num_samples):
            postcode = faker.postcode()
            assert isinstance(postcode, str)
            assert re.fullmatch(r"\d{6}", postcode)

    def test_city_with_postcode(self, faker, num_samples):
        for _ in range(num_samples):
            city_with_postcode = faker.city_with_postcode()
            assert isinstance(city_with_postcode, str)
            match = re.fullmatch(r"\d{6} (?P<city>.*)", city_with_postcode)
            assert match.group("city") in RoRoAddressProvider.cities


class TestEnNz:
    """Test en_NZ address provider methods"""

    def test_te_reo_part(self, faker, num_samples):
        for _ in range(num_samples):
            to_reo_part = faker.te_reo_part()
            assert isinstance(to_reo_part, str)
            assert to_reo_part in EnNzAddressProvider.te_reo_parts

    def test_reo_first(self, faker, num_samples):
        for _ in range(num_samples):
            te_reo_first = faker.te_reo_first()
            assert isinstance(te_reo_first, str)
            assert te_reo_first in [str(i).capitalize() for i in EnNzAddressProvider.te_reo_parts]

    def test_reo_ending(self, faker, num_samples):
        for _ in range(num_samples):
            te_reo_ending = faker.te_reo_ending()
            assert isinstance(te_reo_ending, str)
            assert te_reo_ending in EnNzAddressProvider.te_reo_parts or EnNzAddressProvider.te_reo_endings

    def test_city_prefix(self, faker, num_samples):
        for _ in range(num_samples):
            city_prefix = faker.city_prefix()
            assert isinstance(city_prefix, str)
            assert city_prefix in EnNzAddressProvider.city_prefixes

    def test_city_suffix(self, faker, num_samples):
        for _ in range(num_samples):
            city_suffix = faker.city_suffix()
            assert isinstance(city_suffix, str)
            assert city_suffix in EnNzAddressProvider.city_suffixes

    def test_rd_number(self, faker, num_samples):
        for _ in range(num_samples):
            rd_number = faker.rd_number()
            assert isinstance(rd_number, str)
            assert rd_number in [str(i) for i in range(1, 11)]

    def test_secondary_address(self, faker, num_samples):
        for _ in range(num_samples):
            secondary_address = faker.secondary_address()
            assert isinstance(secondary_address, str)
            assert secondary_address.split(" ")[0] in [
                i.split(" ")[0] for i in EnNzAddressProvider.secondary_address_formats
            ]


class TestFrCh:
    """Test fr_CH address provider methods"""

    def test_canton_name(self, faker, num_samples):
        for _ in range(num_samples):
            canton_name = faker.canton_name()
            assert isinstance(canton_name, str)
            assert any(canton_name == cantons[1] for cantons in FrChAddressProvider.cantons)

    def test_canton_code(self, faker, num_samples):
        for _ in range(num_samples):
            canton_code = faker.canton_code()
            assert isinstance(canton_code, str)
            assert any(canton_code == cantons[0] for cantons in FrChAddressProvider.cantons)

    def test_canton(self, faker, num_samples):
        for _ in range(num_samples):
            canton = faker.canton()
            assert isinstance(canton, tuple)
            assert canton in FrChAddressProvider.cantons


class TestHuHu:
    """Test hu_HU address provider methods"""

    def test_administrative_unit(self, faker, num_samples):
        for _ in range(num_samples):
            administrative_unit = faker.administrative_unit()
            assert isinstance(administrative_unit, str)
            assert administrative_unit in HuHuAddressProvider.counties

    def test_street_address_with_county(self, faker, num_samples):
        for _ in range(num_samples):
            street_address_with_county = faker.street_address_with_county()
            assert isinstance(street_address_with_county, str)
            match = re.fullmatch(r".* \d*.\n.* [A-Za-zÀ-ȕ]*\nH-\d{4} [A-Za-zÀ-ȕ]*", street_address_with_county)
            assert match

    def test_city_prefix(self, faker, num_samples):
        for _ in range(num_samples):
            city_prefix = faker.city_prefix()
            assert isinstance(city_prefix, str)
            assert city_prefix in HuHuAddressProvider.city_prefs

    def test_city_part(self, faker, num_samples):
        for _ in range(num_samples):
            city_part = faker.city_part()
            assert isinstance(city_part, str)
            assert city_part in HuHuAddressProvider.city_parts

    def test_real_city_name(self, faker, num_samples):
        for _ in range(num_samples):
            real_city_name = faker.real_city_name()
            assert isinstance(real_city_name, str)
            assert real_city_name in HuHuAddressProvider.real_city_names

    def test_frequent_street_name(self, faker, num_samples):
        for _ in range(num_samples):
            frequent_street_name = faker.frequent_street_name()
            assert isinstance(frequent_street_name, str)
            assert frequent_street_name in HuHuAddressProvider.frequent_street_names

    def test_postcode(self, faker, num_samples):
        for _ in range(num_samples):
            postcode = faker.postcode()
            assert isinstance(postcode, str)
            match = re.fullmatch(r"H-\d{4}", postcode)
            assert match

    def test_street_name(self, faker, num_samples):
        for _ in range(num_samples):
            street_name = faker.street_name()
            assert isinstance(street_name, str)

    def test_building_number(self, faker, num_samples):
        for _ in range(num_samples):
            building_number = faker.building_number()
            assert isinstance(building_number, str)
            match = re.fullmatch(r"\d{0,3}.", building_number)
            assert match

    def test_city(self, faker, num_samples):
        # generating possible variations for cities for hu_Hu locale
        real_cities = [city.lower() for city in HuHuAddressProvider.real_city_names]
        cities_part_suffix = [
            "".join([part, suffix])
            for part in HuHuAddressProvider.city_parts
            for suffix in HuHuAddressProvider.city_suffixes
        ]
        cities_prefix_part_suffix = [
            "".join([pref, part_suffix])
            for pref in HuHuAddressProvider.city_prefs
            for part_suffix in cities_part_suffix
        ]
        cities = real_cities + cities_part_suffix + cities_prefix_part_suffix
        for _ in range(num_samples):
            city = faker.city()
            assert isinstance(city, str)
            assert city.lower() in cities
            assert city[0].isupper()


class TestIdId:
    """Test id_ID address provider methods"""

    def test_street(self, faker, num_samples):
        for _ in range(num_samples):
            street = faker.street()
            assert isinstance(street, str)
            assert street in IdIdAddressProvider.streets

    def test_street_prefix_short(self, faker, num_samples):
        for _ in range(num_samples):
            street_prefix_short = faker.street_prefix_short()
            assert isinstance(street_prefix_short, str)
            assert street_prefix_short in IdIdAddressProvider.street_prefixes_short

    def test_street_prefix_long(self, faker, num_samples):
        for _ in range(num_samples):
            street_prefix_long = faker.street_prefix_long()
            assert isinstance(street_prefix_long, str)
            assert street_prefix_long in IdIdAddressProvider.street_prefixes_long

    def test_city_name(self, faker, num_samples):
        for _ in range(num_samples):
            city_name = faker.city_name()
            assert isinstance(city_name, str)
            assert city_name in IdIdAddressProvider.cities

    def test_administrative_unit(self, faker, num_samples):
        for _ in range(num_samples):
            administrative_unit = faker.administrative_unit()
            assert isinstance(administrative_unit, str)
            assert administrative_unit in IdIdAddressProvider.states

    def test_state_abbr(self, faker, num_samples):
        for _ in range(num_samples):
            state_abbr = faker.state_abbr()
            assert isinstance(state_abbr, str)
            assert state_abbr in IdIdAddressProvider.states_abbr

    def test_country(self, faker, num_samples):
        for _ in range(num_samples):
            country = faker.country()
            assert isinstance(country, str)
            assert country in IdIdAddressProvider.countries


class TestKaGe:
    """Test ka_GE address provider methods"""

    def test_street_title(self, faker, num_samples):
        for _ in range(num_samples):
            street_title = faker.street_title()
            assert isinstance(street_title, str)
            assert street_title in KaGeAddressProvider.street_titles

    def test_city_name(self, faker, num_samples):
        for _ in range(num_samples):
            city_name = faker.city_name()
            assert isinstance(city_name, str)
            assert city_name in KaGeAddressProvider.city_names


class TestSlSi:
    """Test sl_SI address provider methods"""

    def test_city_name(self, faker, num_samples):
        for _ in range(num_samples):
            city_name = faker.city_name()
            assert isinstance(city_name, str)
            assert city_name in SlSiAddressProvider.cities

    def test_street_name(self, faker, num_samples):
        for _ in range(num_samples):
            street_name = faker.street_name()
            assert isinstance(street_name, str)
            assert street_name in SlSiAddressProvider.streets

    def test_administrative_unit(self, faker, num_samples):
        for _ in range(num_samples):
            administrative_unit = faker.administrative_unit()
            assert isinstance(administrative_unit, str)
            assert administrative_unit in SlSiAddressProvider.states


class TestSvSe:
    """Test sv_SE address provider methods"""

    def test_street_prefix(self, faker, num_samples):
        for _ in range(num_samples):
            street_prefix = faker.street_prefix()
            assert isinstance(street_prefix, str)
            assert street_prefix in SvSeAddressProvider.street_prefixes

    def test_city_name(self, faker, num_samples):
        for _ in range(num_samples):
            city_name = faker.city_name()
            assert isinstance(city_name, str)
            assert city_name in SvSeAddressProvider.cities

    def test_administrative_unit(self, faker, num_samples):
        for _ in range(num_samples):
            administrative_unit = faker.administrative_unit()
            assert isinstance(administrative_unit, str)
            assert administrative_unit in SvSeAddressProvider.states


class TestUkUa:
    """Test uk_UA address provider methods"""

    def test_city_prefix(self, faker, num_samples):
        for _ in range(num_samples):
            city_prefix = faker.city_prefix()
            assert isinstance(city_prefix, str)
            assert city_prefix in UkUaAddressProvider.city_prefixes

    def test_city_name(self, faker, num_samples):
        for _ in range(num_samples):
            city_name = faker.city_name()
            assert isinstance(city_name, str)
            assert city_name in UkUaAddressProvider.city_names

    def test_postcode(self, faker, num_samples):
        for _ in range(num_samples):
            postcode = faker.postcode()
            assert isinstance(postcode, str)
            match = re.findall(r"\d{5}", postcode)
            assert match

    def test_street_prefix(self, faker, num_samples):
        for _ in range(num_samples):
            street_prefix = faker.street_prefix()
            assert isinstance(street_prefix, str)
            assert street_prefix in UkUaAddressProvider.street_prefixes

    def test_street_name(self, faker, num_samples):
        for _ in range(num_samples):
            street_name = faker.street_name()
            assert isinstance(street_name, str)
            assert street_name in UkUaAddressProvider.street_titles

    def test_street_title(self, faker, num_samples):
        for _ in range(num_samples):
            street_title = faker.street_title()
            assert isinstance(street_title, str)

    def test_region(self, faker, num_samples):
        for _ in range(num_samples):
            region = faker.region()
            assert isinstance(region, str)
            assert region in UkUaAddressProvider.region_names


class TestFrCa:
    """Test fr_CA address provider methods"""

    def test_province(self, faker, num_samples):
        for _ in range(num_samples):
            province = faker.province()
            assert isinstance(province, str)
            assert province in FrCaAddressProvider.provinces

    def test_province_abbr(self, faker, num_samples):
        for _ in range(num_samples):
            province_abbr = faker.province_abbr()
            assert isinstance(province_abbr, str)
            assert province_abbr in FrCaAddressProvider.provinces_abbr

    def test_city_prefixes(self, faker, num_samples):
        for _ in range(num_samples):
            city_prefix = faker.city_prefix()
            assert isinstance(city_prefix, str)
            assert city_prefix in FrCaAddressProvider.city_prefixes

    def test_city_suffixes(self, faker, num_samples):
        for _ in range(num_samples):
            city_suffixes = faker.city_suffix()
            assert isinstance(city_suffixes, str)
            assert city_suffixes in FrCaAddressProvider.city_suffixes

    def test_street_prefixes(self, faker, num_samples):
        for _ in range(num_samples):
            street_prefix = faker.street_prefix()
            assert isinstance(street_prefix, str)
            assert street_prefix in FrCaAddressProvider.street_prefixes

    def test_administrative_unit(self, faker, num_samples):
        for _ in range(num_samples):
            province = faker.administrative_unit()
            assert isinstance(province, str)
            assert province in FrCaAddressProvider.provinces


class TestPlPl:
    """Test pl_PL address provider methods"""

    def test_postcode(self, faker, num_samples):
        for _ in range(num_samples):
            postcode = faker.postcode()
            match = re.findall(r"^\d{2}-\d{3}$", postcode)
            assert match

    def test_zipcode(self, faker, num_samples):
        for _ in range(num_samples):
            zipcode = faker.zipcode()
            match = re.findall(r"^\d{2}-\d{3}$", zipcode)
            assert match

    def test_postalcode(self, faker, num_samples):
        for _ in range(num_samples):
            postalcode = faker.postalcode()
            match = re.findall(r"^^\d{2}-\d{3}$$", postalcode)
            assert match



from .. import BaseProvider, ElementsType, date_time

localized = True


class Provider(BaseProvider):
    city_suffixes: ElementsType[str] = ["Ville"]
    street_suffixes: ElementsType[str] = ["Street"]
    city_formats: ElementsType[str] = ("{{first_name}} {{city_suffix}}",)
    street_name_formats: ElementsType[str] = ("{{last_name}} {{street_suffix}}",)
    street_address_formats: ElementsType[str] = ("{{building_number}} {{street_name}}",)
    address_formats: ElementsType[str] = ("{{street_address}} {{postcode}} {{city}}",)
    building_number_formats: ElementsType[str] = ("##",)
    postcode_formats: ElementsType[str] = ("#####",)
    countries: ElementsType[str] = [country.name for country in date_time.Provider.countries]

    ALPHA_2 = "alpha-2"
    ALPHA_3 = "alpha-3"

    alpha_2_country_codes: ElementsType[str] = [country.alpha_2_code for country in date_time.Provider.countries]
    alpha_3_country_codes: ElementsType[str] = [country.alpha_3_code for country in date_time.Provider.countries]

    def city_suffix(self) -> str:
        """
        :example: 'town'
        """
        return self.random_element(self.city_suffixes)

    def street_suffix(self) -> str:
        """
        :example: 'Avenue'
        """
        return self.random_element(self.street_suffixes)

    def building_number(self) -> str:
        """
        :example: '791'
        """
        return self.numerify(self.random_element(self.building_number_formats))

    def city(self) -> str:
        """
        :example: 'Sashabury'
        """
        pattern: str = self.random_element(self.city_formats)
        return self.generator.parse(pattern)

    def street_name(self) -> str:
        """
        :example: 'Crist Parks'
        """
        pattern: str = self.random_element(self.street_name_formats)
        return self.generator.parse(pattern)

    def street_address(self) -> str:
        """
        :example: '791 Crist Parks'
        """
        pattern: str = self.random_element(self.street_address_formats)
        return self.generator.parse(pattern)

    def postcode(self) -> str:
        """
        :example: 86039-9874
        """
        return self.bothify(self.random_element(self.postcode_formats)).upper()

    def address(self) -> str:
        """
        :example: '791 Crist Parks, Sashabury, IL 86039-9874'
        """
        pattern: str = self.random_element(self.address_formats)
        return self.generator.parse(pattern)

    def country(self) -> str:
        return self.random_element(self.countries)

    def country_code(self, representation: str = ALPHA_2) -> str:
        if representation == self.ALPHA_2:
            return self.random_element(self.alpha_2_country_codes)
        elif representation == self.ALPHA_3:
            return self.random_element(self.alpha_3_country_codes)
        else:
            raise ValueError("`representation` must be one of `alpha-2` or `alpha-3`.")

    def current_country_code(self) -> str:
        try:
            return self.__lang__.split("_")[1]  # type: ignore
        except IndexError:
            raise AttributeError("Country code cannot be determined from locale")

    def current_country(self) -> str:
        current_country_code = self.current_country_code()
        current_country = [
            country.name for country in date_time.Provider.countries if country.alpha_2_code == current_country_code
        ]
        if len(current_country) == 1:
            return current_country[0]  # type: ignore
        elif len(current_country) > 1:
            raise ValueError(f"Ambiguous country for country code {current_country_code}: {current_country}")
        else:
            raise ValueError(f"No appropriate country for country code {current_country_code}")



from .. import Provider as PersonProvider


class Provider(PersonProvider):
    formats_male = (
        "{{first_name_male}} {{last_name}}",
        "{{first_name_male}} {{last_name}}",
        "{{first_name_male}} {{last_name}}",
        "{{first_name_male}} {{last_name}}",
        "{{first_name_male}} {{last_name}}",
        "{{first_name_male}} {{last_name}}",
        "{{first_name_male}} {{last_name}}",
        "{{first_name_male}} {{prefix}} {{last_name}}",
        "{{first_name_male}} {{last_name}}-{{last_name}}",
    )

    formats_female = (
        "{{first_name_female}} {{last_name}}",
        "{{first_name_female}} {{last_name}}",
        "{{first_name_female}} {{last_name}}",
        "{{first_name_female}} {{last_name}}",
        "{{first_name_female}} {{last_name}}",
        "{{first_name_female}} {{last_name}}",
        "{{first_name_female}} {{last_name}}",
        "{{first_name_female}} {{prefix}} {{last_name}}",
        "{{first_name_female}} {{last_name}}-{{last_name}}",
        "{{first_name_female}}-{{first_name_female}} {{last_name}}",
    )

    formats = formats_male + formats_female

    first_names_male = (
        "Afonso",
        "Alexandre",
        "Álvaro",
        "André",
        "Ângelo",
        "António",
        "Artur",
        "Benjamim",
        "Bernardo",
        "Brian",
        "Bruno",
        "Bryan",
        "Carlos",
        "Cláudio",
        "Cristiano",
        "César",
        "Daniel",
        "David",
        "Denis",
        "Diego",
        "Dinis",
        "Diogo",
        "Duarte",
        "Edgar",
        "Eduardo",
        "Emanuel",
        "Enzo",
        "Fernando",
        "Filipe",
        "Francisco",
        "Frederico",
        "Fábio",
        "Gabriel",
        "Gaspar",
        "Gil",
        "Gonçalo",
        "Guilherme",
        "Gustavo",
        "Henrique",
        "Hugo",
        "Igor",
        "Isaac",
        "Ismael",
        "Ivan",
        "Ivo",
        "Jaime",
        "Joaquim",
        "Joel",
        "Jorge",
        "José",
        "João",
        "Kevin",
        "Kévim",
        "Leandro",
        "Leonardo",
        "Lisandro",
        "Lourenço",
        "Luca",
        "Lucas",
        "Luís",
        "Manuel",
        "Marco",
        "Marcos",
        "Martim",
        "Mateus",
        "Matias",
        "Mauro",
        "Micael",
        "Miguel",
        "Márcio",
        "Mário",
        "Nelson",
        "Noa",
        "Noah",
        "Nuno",
        "Paulo",
        "Pedro",
        "Rafael",
        "Renato",
        "Ricardo",
        "Rodrigo",
        "Rui",
        "Rúben",
        "Salvador",
        "Samuel",
        "Sandro",
        "Santiago",
        "Sebastião",
        "Simão",
        "Sérgio",
        "Tiago",
        "Tomás",
        "Tomé",
        "Valentim",
        "Vasco",
        "Vicente",
        "Vítor",
        "William",
        "Wilson",
        "Xavier",
    )

    first_names_female = (
        "Adriana",
        "Alexandra",
        "Alice",
        "Alícia",
        "Amélia",
        "Ana",
        "Andreia",
        "Ângela",
        "Anita",
        "Ariana",
        "Beatriz",
        "Benedita",
        "Bianca",
        "Bruna",
        "Bárbara",
        "Caetana",
        "Camila",
        "Carlota",
        "Carminho",
        "Carolina",
        "Catarina",
        "Clara",
        "Constança",
        "Daniela",
        "Diana",
        "Débora",
        "Eduarda",
        "Ema",
        "Emma",
        "Emília",
        "Erica",
        "Érica",
        "Erika",
        "Eva",
        "Fabiana",
        "Filipa",
        "Flor",
        "Francisca",
        "Gabriela",
        "Helena",
        "Iara",
        "Inês",
        "Irina",
        "Íris",
        "Isabel",
        "Isabela",
        "Joana",
        "Juliana",
        "Jéssica",
        "Júlia",
        "Kelly",
        "Kyara",
        "Lara",
        "Larissa",
        "Laura",
        "Leonor",
        "Letícia",
        "Lia",
        "Lorena",
        "Luana",
        "Luciana",
        "Luna",
        "Luísa",
        "Lúcia",
        "Madalena",
        "Mafalda",
        "Mara",
        "Margarida",
        "Maria",
        "Mariana",
        "Marta",
        "Matilde",
        "Melissa",
        "Mia",
        "Miriam",
        "Mélanie",
        "Naiara",
        "Nair",
        "Nicole",
        "Nádia",
        "Núria",
        "Patrícia",
        "Petra",
        "Pilar",
        "Rafaela",
        "Raquel",
        "Renata",
        "Rita",
        "Salomé",
        "Sara",
        "Sofia",
        "Soraia",
        "Tatiana",
        "Teresa",
        "Valentina",
        "Vera",
        "Victória",
        "Violeta",
        "Vitória",
        "Yara",
        "Yasmin",
    )

    first_names = first_names_male + first_names_female

    last_names = (
        "Abreu",
        "Almeida",
        "Alves",
        "Amaral",
        "Amorim",
        "Andrade",
        "Anjos",
        "Antunes",
        "Araújo",
        "Assunção",
        "Azevedo",
        "Baptista",
        "Barbosa",
        "Barros",
        "Batista",
        "Borges",
        "Branco",
        "Brito",
        "Campos",
        "Cardoso",
        "Carneiro",
        "Carvalho",
        "Castro",
        "Coelho",
        "Correia",
        "Costa",
        "Cruz",
        "Cunha",
        "Domingues",
        "Esteves",
        "Faria",
        "Fernandes",
        "Ferreira",
        "Figueiredo",
        "Fonseca",
        "Freitas",
        "Garcia",
        "Gaspar",
        "Gomes",
        "Gonçalves",
        "Guerreiro",
        "Henriques",
        "Jesus",
        "Leal",
        "Leite",
        "Lima",
        "Lopes",
        "Loureiro",
        "Lourenço",
        "Macedo",
        "Machado",
        "Magalhães",
        "Maia",
        "Marques",
        "Martins",
        "Matias",
        "Matos",
        "Melo",
        "Mendes",
        "Miranda",
        "Monteiro",
        "Morais",
        "Moreira",
        "Mota",
        "Moura",
        "Nascimento",
        "Neto",
        "Neves",
        "Nogueira",
        "Nunes",
        "Oliveira",
        "Pacheco",
        "Paiva",
        "Pereira",
        "Pinheiro",
        "Pinho",
        "Pinto",
        "Pires",
        "Ramos",
        "Reis",
        "Ribeiro",
        "Rocha",
        "Rodrigues",
        "Santos",
        "Silva",
        "Simões",
        "Soares",
        "Sousa",
        "Sá",
        "Tavares",
        "Teixeira",
        "Torres",
        "Valente",
        "Vaz",
        "Vicente",
        "Vieira",
    )

    prefixes = ("de", "da", "do")

    def prefix(self) -> str:
        return self.random_element(self.prefixes)



"""TEST NOTES:

The following tests cover cases where a ``faker_session_locale`` fixture was not
defined by the user, but non-autouse ``faker_locale`` and ``faker_seed`` fixtures
were defined. The resulting behavior of the ``faker`` fixture will vary depending
on which fixtures are injected.
"""


from random import Random

import pytest

from faker.contrib.pytest.plugin import DEFAULT_LOCALE, DEFAULT_SEED


@pytest.fixture()
def faker_locale():
    return ["it_IT"]


@pytest.fixture()
def faker_seed():
    return 4761


def test_no_injection(_session_faker, faker):
    random = Random(DEFAULT_SEED)
    assert faker == _session_faker
    assert faker.locales == [DEFAULT_LOCALE]
    assert faker.random != random
    assert faker.random.getstate() == random.getstate()


def test_inject_faker_locale(_session_faker, faker, faker_locale):
    random = Random(DEFAULT_SEED)
    assert faker != _session_faker
    assert faker.locales == faker_locale
    assert faker.random != random
    assert faker.random.getstate() == random.getstate()


def test_inject_faker_seed(_session_faker, faker, faker_seed):
    random = Random(faker_seed)
    assert faker == _session_faker
    assert faker.locales == [DEFAULT_LOCALE]
    assert faker.random != random
    assert faker.random.getstate() == random.getstate()


def test_inject_faker_seed_and_locale(_session_faker, faker, faker_locale, faker_seed):
    random = Random(faker_seed)
    assert faker != _session_faker
    assert faker.locales == faker_locale
    assert faker.random != random
    assert faker.random.getstate() == random.getstate()



import decimal
import sys
import unittest
import warnings

from typing import Iterable, Optional, Union
from unittest.mock import patch

import pytest

from faker import Faker


@pytest.mark.parametrize("object_type", (None, bool, str, float, int, tuple, set, list, Iterable, dict))
def test_pyobject(
    object_type: Optional[Union[bool, str, float, int, tuple, set, list, Iterable, dict]],
):
    random_object = Faker().pyobject(object_type=object_type)
    if object_type is None:
        assert random_object is None
    else:
        assert isinstance(random_object, object_type)


@pytest.mark.parametrize("object_type", (object, type, callable))
def test_pyobject_with_unknown_object_type(object_type):
    with pytest.raises(ValueError, match=f"Object type `{object_type}` is not supported by `pyobject` function"):
        assert Faker().pyobject(object_type=object_type)


@pytest.mark.parametrize(
    "mock_random_number_source, right_digits, expected_decimal_part",
    (
        ("1234567", 5, "12345"),
        ("1234567", 0, "1"),  # This is kinda interesting - same as 1 digit
        ("1234567", 1, "1"),
        ("1234567", 2, "12"),
        ("0123", 1, "1"),
    ),
)
def test_pyfloat_right_and_left_digits_positive(mock_random_number_source, right_digits, expected_decimal_part):
    # Remove the randomness from the test by mocking the `BaseProvider.random_number` value
    def mock_random_number(self, digits=None, fix_len=False):
        return int(mock_random_number_source[: digits or 1])

    with patch("faker.providers.BaseProvider.random_number", mock_random_number):
        result = Faker().pyfloat(left_digits=1, right_digits=right_digits, positive=True)
        decimal_part = str(result).split(".")[1]
        assert decimal_part == expected_decimal_part


def test_pyfloat_right_or_left_digit_overflow():
    max_float_digits = sys.float_info.dig
    faker = Faker()

    # Make random_int always return the maximum value input - makes it easy to reason about the code below
    def mock_random_int(self, min=0, max=9999, step=1):
        return max

    # Remove the randomness from the test by mocking the `BaseProvider.random_number` value
    def mock_random_number(self, digits=None, fix_len=False):
        return int("12345678901234567890"[: digits or 1])

    with patch("faker.providers.BaseProvider.random_int", mock_random_int):
        with patch("faker.providers.BaseProvider.random_number", mock_random_number):
            # A bit too much, but ~half on either side
            with pytest.raises(ValueError, match="Asking for too many digits"):
                faker.pyfloat(
                    left_digits=max_float_digits // 2 + 1,
                    right_digits=max_float_digits // 2 + 1,
                )

            # Asking for max digits on either side also fails, because we need one digit on the other side, i.e.
            # 0.123123123, or 123123123.0 (at least needs to lead with `0.` or trail with `.0`).
            with pytest.raises(ValueError, match="Asking for too many digits"):
                faker.pyfloat(left_digits=max_float_digits)
            with pytest.raises(ValueError, match="Asking for too many digits"):
                faker.pyfloat(right_digits=max_float_digits)

            # Just the right amount of max digits on either side
            result = faker.pyfloat(left_digits=max_float_digits - 1)
            assert str(abs(result)) == "12345678901234.1"
            result = faker.pyfloat(right_digits=max_float_digits - 1)
            assert str(abs(result)) == "1.12345678901234"


@pytest.mark.skipif(sys.version_info < (3, 10), reason="Only relevant for Python 3.10 and later.")
@pytest.mark.parametrize(
    ("min_value", "max_value"),
    [
        (1.5, None),
        (-1.5, None),
        (None, -1.5),
        (None, 1.5),
        (-1.5, 1.5),
    ],
)
@pytest.mark.parametrize(("left_digits"), [None, 5])
@pytest.mark.parametrize(("right_digits"), [None, 5])
@pytest.mark.filterwarnings(
    # Convert the warning to an error for this test
    r"error:non-integer arguments to randrange\(\):DeprecationWarning"
)
def test_float_min_and_max_value_does_not_crash(
    left_digits: Optional[int],
    right_digits: Optional[int],
    min_value: Optional[float],
    max_value: Optional[float],
):
    """
    Float arguments to randrange are deprecated from Python 3.10. This is a regression
    test to check that `pydecimal` does not cause a crash on any code path.
    """
    Faker().pydecimal(left_digits, right_digits, min_value=min_value, max_value=max_value)


class TestPyint(unittest.TestCase):
    def setUp(self):
        self.fake = Faker()
        Faker.seed(0)

    def test_pyint(self):
        self.assertIsInstance(self.fake.pyint(), int)

    def test_pyint_bounds(self):
        self.assertTrue(0 <= self.fake.pyint() <= 9999)

    def test_pyint_step(self):
        random_int = self.fake.pyint(step=2)
        self.assertEqual(0, random_int % 2)

    def test_pyint_bound_0(self):
        self.assertEqual(0, self.fake.pyint(min_value=0, max_value=0))

    def test_pyint_bound_positive(self):
        self.assertEqual(5, self.fake.pyint(min_value=5, max_value=5))

    def test_pyint_bound_negative(self):
        self.assertEqual(-5, self.fake.pyint(min_value=-5, max_value=-5))

    def test_pyint_range(self):
        self.assertTrue(0 <= self.fake.pyint(min_value=0, max_value=2) <= 2)


class TestPyfloat(unittest.TestCase):
    def setUp(self):
        self.fake = Faker()
        Faker.seed(0)

    def test_pyfloat(self):
        result = self.fake.pyfloat()

        self.assertIsInstance(result, float)

    def test_left_digits(self):
        expected_left_digits = 10

        result = self.fake.pyfloat(left_digits=expected_left_digits)

        left_digits = len(str(abs(int(result))))
        self.assertGreaterEqual(expected_left_digits, left_digits)

    def test_right_digits(self):
        expected_right_digits = 10

        result = self.fake.pyfloat(right_digits=expected_right_digits)

        right_digits = len(("%r" % result).split(".")[1])
        self.assertGreaterEqual(expected_right_digits, right_digits)

    def test_positive(self):
        result = self.fake.pyfloat(positive=True)

        self.assertGreater(result, 0)
        self.assertEqual(result, abs(result))

    def test_min_value(self):
        min_values = (0, 10, -1000, 1000, 999999)

        for min_value in min_values:
            result = self.fake.pyfloat(min_value=min_value)
            self.assertGreaterEqual(result, min_value)

    def test_min_value_and_left_digits(self):
        """
        Combining the min_value and left_digits keyword arguments produces
        numbers that obey both of those constraints.
        """

        result = self.fake.pyfloat(left_digits=1, min_value=0)
        self.assertLess(result, 10)
        self.assertGreaterEqual(result, 0)

    def test_max_value(self):
        max_values = (0, 10, -1000, 1000, 999999)

        for max_value in max_values:
            result = self.fake.pyfloat(max_value=max_value)
            self.assertLessEqual(result, max_value)

    def test_max_value_zero_and_left_digits(self):
        """
        Combining the max_value and left_digits keyword arguments produces
        numbers that obey both of those constraints.
        """

        result = self.fake.pyfloat(left_digits=2, max_value=0)
        self.assertLessEqual(result, 0)
        self.assertGreater(result, -100)

    def test_max_value_should_be_greater_than_min_value(self):
        """
        An exception should be raised if min_value is greater than max_value
        """
        expected_message = "Min value cannot be greater than max value"
        with self.assertRaises(ValueError) as raises:
            self.fake.pyfloat(min_value=100, max_value=0)

        message = str(raises.exception)
        self.assertEqual(message, expected_message)

    def test_max_value_and_positive(self):
        """
        Combining the max_value and positive keyword arguments produces
        numbers that obey both of those constraints.
        """

        result = self.fake.pyfloat(positive=True, max_value=100)
        self.assertLessEqual(result, 100)
        self.assertGreater(result, 0)

    def test_max_and_min_value_positive_with_decimals(self):
        """
        Combining the max_value and min_value keyword arguments with
        positive values for each produces numbers that obey both of
        those constraints.
        """
        for _ in range(1000):
            result = self.fake.pyfloat(min_value=100.123, max_value=200.321)
            self.assertLessEqual(result, 200.321)
            self.assertGreaterEqual(result, 100.123)

    def test_max_and_min_value_negative(self):
        """
        Combining the max_value and min_value keyword arguments with
        negative values for each produces numbers that obey both of
        those constraints.
        """

        result = self.fake.pyfloat(max_value=-100, min_value=-200)
        self.assertLessEqual(result, -100)
        self.assertGreaterEqual(result, -200)

    def test_max_and_min_value_negative_with_decimals(self):
        """
        Combining the max_value and min_value keyword arguments with
        negative values for each produces numbers that obey both of
        those constraints.
        """
        for _ in range(1000):
            result = self.fake.pyfloat(max_value=-100.123, min_value=-200.321)
            self.assertLessEqual(result, -100.123)
            self.assertGreaterEqual(result, -200.321)

    def test_positive_and_min_value_incompatible(self):
        """
        An exception should be raised if positive=True is set, but
        a negative min_value is provided.
        """

        expected_message = "Cannot combine positive=True with negative or zero min_value"
        with self.assertRaises(ValueError) as raises:
            self.fake.pyfloat(min_value=-100, positive=True)

        message = str(raises.exception)
        self.assertEqual(message, expected_message)

    def test_positive_doesnt_return_zero(self):
        """
        Choose the right_digits and max_value so it's guaranteed to return zero,
        then watch as it doesn't because positive=True
        """
        result = self.fake.pyfloat(positive=True, right_digits=0, max_value=1)
        self.assertGreater(result, 0)

    @pytest.mark.skipif(sys.version_info < (3, 10), reason="Only relevant for Python 3.10 and later.")
    @pytest.mark.filterwarnings(
        # Convert the warning to an error for this test
        r"error:non-integer arguments to randrange\(\):DeprecationWarning"
    )
    def test_float_min_and_max_value_does_not_warn(self):
        """
        Float arguments to randrange are deprecated from Python 3.10. This is a regression
        test to check that `pyfloat` does not cause a deprecation warning.
        """
        self.fake.pyfloat(min_value=-1.0, max_value=1.0)

    def test_float_min_and_max_value_with_same_whole(self):
        self.fake.pyfloat(min_value=2.3, max_value=2.5)


class TestPydecimal(unittest.TestCase):
    def setUp(self):
        self.fake = Faker()
        Faker.seed(0)

    def test_pydecimal(self):
        result = self.fake.pydecimal()

        self.assertIsInstance(result, decimal.Decimal)

    def test_left_digits(self):
        expected_left_digits = 10

        result = self.fake.pydecimal(left_digits=expected_left_digits)

        left_digits = len(str(abs(int(result))))
        self.assertGreaterEqual(expected_left_digits, left_digits)

    def test_left_digits_can_be_zero(self):
        expected_left_digits = 0

        result = self.fake.pydecimal(left_digits=expected_left_digits)

        left_digits = int(result)
        self.assertEqual(expected_left_digits, left_digits)

    def test_right_digits(self):
        expected_right_digits = 10

        result = self.fake.pydecimal(right_digits=expected_right_digits)

        right_digits = len(str(result).split(".")[1])
        self.assertGreaterEqual(expected_right_digits, right_digits)

    def test_positive(self):
        result = self.fake.pydecimal(positive=True)

        self.assertGreater(result, 0)
        abs_result = -result if result < 0 else result  # abs() result returns scientific notation
        self.assertEqual(result, abs_result)

    def test_min_value(self):
        min_values = (0, 10, -1000, 1000, 999999)

        for min_value in min_values:
            result = self.fake.pydecimal(min_value=min_value)
            self.assertGreaterEqual(result, min_value)

    def test_min_value_always_returns_a_decimal(self):
        min_values = (0, 10, -1000, 1000, 999999)

        for min_value in min_values:
            result = self.fake.pydecimal(min_value=min_value)
            self.assertIsInstance(result, decimal.Decimal)

    def test_min_value_and_left_digits(self):
        """
        Combining the min_value and left_digits keyword arguments produces
        numbers that obey both of those constraints.
        """

        result = self.fake.pydecimal(left_digits=1, min_value=0)
        self.assertLess(result, 10)
        self.assertGreaterEqual(result, 0)

    def test_max_value(self):
        max_values = (0, 10, -1000, 1000, 999999)

        for max_value in max_values:
            result = self.fake.pydecimal(max_value=max_value)
            self.assertLessEqual(result, max_value)

    def test_max_value_always_returns_a_decimal(self):
        max_values = (0, 10, -1000, 1000, 999999)

        for max_value in max_values:
            result = self.fake.pydecimal(max_value=max_value)
            self.assertIsInstance(result, decimal.Decimal)

    def test_max_value_zero_and_left_digits(self):
        """
        Combining the max_value and left_digits keyword arguments produces
        numbers that obey both of those constraints.
        """

        result = self.fake.pydecimal(left_digits=2, max_value=0)
        self.assertLessEqual(result, 0)
        self.assertGreater(result, -100)

    def test_max_value_should_be_greater_than_min_value(self):
        """
        An exception should be raised if min_value is greater than max_value
        """
        expected_message = "Min value cannot be greater than max value"
        with self.assertRaises(ValueError) as raises:
            self.fake.pydecimal(min_value=100, max_value=0)

        message = str(raises.exception)
        self.assertEqual(message, expected_message)

    def test_max_value_and_positive(self):
        """
        Combining the max_value and positive keyword arguments produces
        numbers that obey both of those constraints.
        """

        result = self.fake.pydecimal(positive=True, max_value=100)
        self.assertLessEqual(result, 100)
        self.assertGreater(result, 0)

    def test_max_and_min_value_negative(self):
        """
        Combining the max_value and min_value keyword arguments with
        negative values for each produces numbers that obey both of
        those constraints.
        """

        result = self.fake.pydecimal(max_value=-100, min_value=-200)
        self.assertLessEqual(result, -100)
        self.assertGreaterEqual(result, -200)

    def test_positive_and_min_value_incompatible(self):
        """
        An exception should be raised if positive=True is set, but
        a negative min_value is provided.
        """

        expected_message = "Cannot combine positive=True with negative or zero min_value"
        with self.assertRaises(ValueError) as raises:
            self.fake.pydecimal(min_value=-100, positive=True)

        message = str(raises.exception)
        self.assertEqual(message, expected_message)

    def test_positive_doesnt_return_zero(self):
        """
        Choose the right_digits and max_value so it's guaranteed to return zero,
        then watch as it doesn't because positive=True
        """
        result = self.fake.pydecimal(positive=True, right_digits=0, max_value=1)
        self.assertGreater(result, 0)

    def test_min_value_zero_doesnt_return_negative(self):
        Faker.seed("1")
        result = self.fake.pydecimal(left_digits=3, right_digits=2, min_value=0, max_value=999)
        self.assertGreater(result, 0)

    def test_min_value_one_hundred_doesnt_return_negative(self):
        Faker.seed("1")
        result = self.fake.pydecimal(left_digits=3, right_digits=2, min_value=100, max_value=999)
        self.assertGreater(result, 100)

    def test_min_value_minus_one_doesnt_return_positive(self):
        Faker.seed("5")
        result = self.fake.pydecimal(left_digits=3, right_digits=2, min_value=-999, max_value=0)
        self.assertLess(result, 0)

    def test_min_value_minus_one_hundred_doesnt_return_positive(self):
        Faker.seed("5")
        result = self.fake.pydecimal(left_digits=3, right_digits=2, min_value=-999, max_value=-100)
        self.assertLess(result, -100)

    def test_min_value_10_pow_1000_return_greater_number(self):
        Faker.seed("2")
        result = self.fake.pydecimal(min_value=10**1000)
        self.assertGreater(result, 10**1000)


class TestPystr(unittest.TestCase):
    def setUp(self):
        self.fake = Faker(includes=["tests.mymodule.en_US"])
        Faker.seed(0)

    def test_no_parameters(self):
        some_string = self.fake.pystr()
        assert isinstance(some_string, str)
        assert len(some_string) <= 20

    def test_lower_length_limit(self):
        some_string = self.fake.pystr(min_chars=3)
        assert isinstance(some_string, str)
        assert len(some_string) >= 3
        assert len(some_string) <= 20

    def test_upper_length_limit(self):
        some_string = self.fake.pystr(max_chars=5)
        assert isinstance(some_string, str)
        assert len(some_string) <= 5

    def test_invalid_length_limits(self):
        with self.assertRaises(AssertionError):
            self.fake.pystr(min_chars=6, max_chars=5)

    def test_exact_length(self):
        some_string = self.fake.pystr(min_chars=5, max_chars=5)
        assert isinstance(some_string, str)
        assert len(some_string) == 5

    def test_prefix(self):
        some_string = self.fake.pystr(prefix="START_")
        assert isinstance(some_string, str)
        assert some_string.startswith("START_")
        assert len(some_string) == 26

    def test_suffix(self):
        some_string = self.fake.pystr(suffix="_END")
        assert isinstance(some_string, str)
        assert some_string.endswith("_END")
        assert len(some_string) == 24

    def test_prefix_and_suffix(self):
        some_string = self.fake.pystr(min_chars=9, max_chars=20, prefix="START_", suffix="_END")
        assert isinstance(some_string, str)
        assert some_string.startswith("START_")
        assert some_string.endswith("_END")
        assert len(some_string) >= 19


class TestPystrFormat(unittest.TestCase):
    def setUp(self):
        self.fake = Faker(includes=["tests.mymodule.en_US"])
        Faker.seed(0)

    def test_formatter_invocation(self):
        with patch.object(self.fake["en_US"].factories[0], "foo") as mock_foo:
            with patch("faker.providers.BaseProvider.bothify", wraps=self.fake.bothify) as mock_bothify:
                mock_foo.return_value = "barbar"
                value = self.fake.pystr_format("{{foo}}?#?{{foo}}?#?{{foo}}", letters="abcde")
                assert value.count("barbar") == 3
                assert mock_foo.call_count == 3
                mock_bothify.assert_called_once_with("barbar?#?barbar?#?barbar", letters="abcde")


class TestPython(unittest.TestCase):
    """Tests python generators"""

    def setUp(self):
        self.factory = Faker()

    def test_pybool_return_type(self):
        some_bool = self.factory.pybool()
        assert isinstance(some_bool, bool)

    def __test_pybool_truth_probability(
        self,
        truth_probability: int,
        deviation_threshold: int = 5,
        iterations: int = 999,
    ):
        truth_count_expected = iterations * truth_probability / 100
        truth_count_actual = 0

        for iteration in range(iterations):
            boolean = self.factory.pybool(truth_probability=truth_probability)
            assert isinstance(boolean, bool)
            if boolean is True:
                truth_count_actual += 1

        deviation_absolute = abs(truth_count_expected - truth_count_actual)
        deviation_percentage = deviation_absolute / iterations * 100

        # Increase `deviation_threshold` value in case this assertion becomes flaky.
        assert deviation_percentage <= deviation_threshold

    def test_pybool_truth_probability_zero(self):
        self.__test_pybool_truth_probability(0, deviation_threshold=0)

    def test_pybool_truth_probability_twenty_five(self):
        self.__test_pybool_truth_probability(25)

    def test_pybool_truth_probability_fifty(self):
        self.__test_pybool_truth_probability(50)

    def test_pybool_truth_probability_seventy_five(self):
        self.__test_pybool_truth_probability(75)

    def test_pybool_truth_probability_hundred(self):
        self.__test_pybool_truth_probability(100, deviation_threshold=0)

    def __test_pybool_invalid_truth_probability(self, truth_probability: int):
        with pytest.raises(ValueError) as exception:
            self.factory.pybool(truth_probability=truth_probability)

        message_expected = "Invalid `truth_probability` value: must be between `0` and `100` inclusive"
        message_actual = str(exception.value)
        assert message_expected == message_actual

    def test_pybool_truth_probability_less_than_zero(self):
        self.__test_pybool_invalid_truth_probability(-1)

    def test_pybool_truth_probability_more_than_hundred(self):
        self.__test_pybool_invalid_truth_probability(101)

    def test_pytuple(self):
        with warnings.catch_warnings(record=True) as w:
            some_tuple = Faker().pytuple()
            assert len(w) == 0
        assert some_tuple
        assert isinstance(some_tuple, tuple)

    def test_pytuple_size(self):
        def mock_pyint(self, *args, **kwargs):
            return 1

        with patch("faker.providers.python.Provider.pyint", mock_pyint):
            some_tuple = Faker().pytuple(nb_elements=3, variable_nb_elements=False, value_types=[int])
            assert some_tuple == (1, 1, 1)

    def test_pylist(self):
        with warnings.catch_warnings(record=True) as w:
            some_list = self.factory.pylist()
            assert len(w) == 0
        assert some_list
        assert isinstance(some_list, list)

    def test_pylist_types(self):
        with warnings.catch_warnings(record=True) as w:
            some_list = self.factory.pylist(10, True, [int])
            assert len(w) == 0
        assert some_list
        for item in some_list:
            assert isinstance(item, int)

        with warnings.catch_warnings(record=True) as w:
            some_list = self.factory.pylist(10, True, value_types=[int])
            assert len(w) == 0
        assert some_list
        for item in some_list:
            assert isinstance(item, int)

        with warnings.catch_warnings(record=True) as w:
            some_list = self.factory.pylist(10, True, int)
            assert len(w) == 1
        assert some_list
        for item in some_list:
            assert isinstance(item, int)

        with warnings.catch_warnings(record=True) as w:
            some_list = self.factory.pylist(10, True, int, float)
            assert len(w) == 2
        assert some_list
        for item in some_list:
            assert isinstance(item, (int, float))



from collections import OrderedDict

from .. import Provider as InternetProvider


class Provider(InternetProvider):
    free_email_domains = (
        "hotmail.com",
        "gmail.com",
        "outlook.com",
        "yahoo.com",
        "ymail.com",
        "kon.in.th",
        "icloud.com",
        "protonmail.com",
    )

    tlds = OrderedDict(
        (
            ("in.th", 100),
            ("co.th", 80),
            ("go.th", 40),
            ("or.th", 40),
            ("ac.th", 20),
            ("net.th", 10),
            ("mi.th", 5),
            ("co", 10),
            ("net", 20),
            ("com", 150),
            ("org", 50),
        ),
    )



"""TEST NOTES:

The following tests cover cases where a ``faker_session_locale`` fixture was not
defined by the user, but an autouse ``faker_locale`` fixture was defined. In this
setup, the plugin's session level ``Faker`` instance will not be used, and each
test will instead generate a new instance using the value of ``faker_locale``.
These new instances will be still seeded in accordance to the plugin's seeding
rules.
"""

from random import Random

import pytest

from faker.contrib.pytest.plugin import DEFAULT_SEED

_CHANGED_LOCALE = ["it_IT"]


@pytest.fixture(autouse=True)
def faker_locale():
    return _CHANGED_LOCALE


@pytest.fixture()
def faker_seed():
    return 4761


def test_no_injection(_session_faker, faker):
    random = Random(DEFAULT_SEED)
    assert faker != _session_faker
    assert faker.locales == _CHANGED_LOCALE
    assert faker.random != random
    assert faker.random.getstate() == random.getstate()


def test_inject_faker_seed(_session_faker, faker, faker_seed):
    random = Random(faker_seed)
    assert faker != _session_faker
    assert faker.locales == _CHANGED_LOCALE
    assert faker.random != random
    assert faker.random.getstate() == random.getstate()



from .. import Provider as AutomotiveProvider


class Provider(AutomotiveProvider):
    """Implement automotive provider for ``ar_PS`` locale.

    Sources:

    - https://en.wikipedia.org/wiki/Vehicle_registration_plates_of_the_Palestinian_National_Authority
    """

    license_formats = (
        # Private vehicles
        "{{district}}-####-3#",
        "{{district}}-####-4#",
        "{{district}}-####-7#",
        "{{district}}-####-9#",
        # Public transport
        "{{district}}-####-30",
        # Authority vehicles
        "####",
        # New police vehicles
        "####-99",
        # Gaza strip after 2012
        # Private
        "1-####-0#",
        "3-####-0#",
        # Commercial
        "1-####-1#",
        "3-####-1#",
        # Public
        "1-####-2#",
        "3-####-2#",
        # Municipal
        "1-####-4#",
        "3-####-4#",
        # Governmental, and Governmental personal vehicles
        "1-####-5#",
        "3-####-5#",
    )

    def district(self) -> str:
        """Generate a district code for license plates."""
        return self.random_element(
            [
                # Gaza Strip
                "1",
                "3",
                # Northern West Bank (Nablus, Tulkarm, Qalqilya, Jenin)
                "4",
                "7",
                # Central West Bank (Ramallah, Jerusalem, Jericho)
                "5",
                "6",
                # Southern West Bank (Bethlehem, Hebron)
                "8",
                "9",
            ]
        )

    def license_plate(self) -> str:
        """Generate a license plate."""
        pattern: str = self.random_element(self.license_formats)
        return self.numerify(self.generator.parse(pattern))



import dataclasses
import sys

from datetime import date, datetime, timedelta
from typing import Sequence, Union

try:
    from typing import List, Literal, TypeVar  # type: ignore
except ImportError:
    from typing import List

    from typing_extensions import Literal, TypeVar  # type: ignore

if sys.version_info >= (3, 9):
    from collections import OrderedDict as OrderedDictType
else:
    from typing import OrderedDict as OrderedDictType


class CreditCard:
    def __init__(
        self,
        name: str,
        prefixes: List[str],
        length: int = 16,
        security_code: str = "CVC",
        security_code_length: int = 3,
    ) -> None:
        self.name = name
        self.prefixes = prefixes
        self.length = length
        self.security_code = security_code
        self.security_code_length = security_code_length


CardType = TypeVar("CardType", "CreditCard", str)
DateParseType = Union[date, datetime, timedelta, str, int]
HueType = Union[str, float, int, Sequence[int]]
SexLiteral = Literal["M", "F"]
SeedType = Union[int, float, str, bytes, bytearray, None]


@dataclasses.dataclass
class Country:
    name: str
    timezones: Sequence[str]
    alpha_2_code: str
    alpha_3_code: str
    continent: str
    capital: str


__all__ = ["OrderedDictType", "CreditCard", "CardType", "Country", "DateParseType", "HueType", "SexLiteral", "SeedType"]



from .. import Provider as CompanyProvider


class Provider(CompanyProvider):
    formats = (
        "{{last_name}} {{company_suffix}}",
        "{{last_name}} {{last_name}} {{company_suffix}}",
        "{{last_name}} {{last_name}} {{company_suffix}}",
        "{{last_name}}",
    )

    company_suffixes = (
        "As Oy",
        "Tmi",
        "Oy",
        "Oyj",
        "Ky",
        "Osk",
        "ry",
    )

    def company_business_id(self) -> str:
        """
        Returns Finnish company Business Identity Code (y-tunnus).
        Format is 8 digits - e.g. FI99999999,[8] last digit is a check
        digit utilizing MOD 11-2. The first digit is zero for some old
        organizations. This function provides current codes starting with
        non-zero.
        """

        def calculate_checksum(number: str) -> str:
            """Calculate the checksum using mod 11,2 method"""
            factors = [7, 9, 10, 5, 8, 4, 2]
            sum_ = 0
            for x, y in zip(number, factors):
                sum_ = sum_ + int(x) * y
            if sum_ % 11 == 1:
                raise ValueError("Checksum 1 is invalid")
            if sum_ % 11 == 0:
                return "0"
            else:
                return str(11 - sum_ % 11)

        while True:
            first_digit = str(self.random_digit_not_null())
            body = first_digit + self.bothify("######")
            try:
                cs = calculate_checksum(body)
            except ValueError:
                continue
            return body + "-" + str(cs)

    def company_vat(self) -> str:
        """
        Returns Finnish VAT identification number (Arvonlisaveronumero).
        This can be calculated from company business identity code by
        adding prefix "FI" and removing dash before checksum.
        """

        def convert_to_vat(business_id: str) -> str:
            """
            Convert business id to VATIN
            """
            return "FI" + business_id.replace("-", "")

        return convert_to_vat(self.company_business_id())



from collections import OrderedDict

from .. import Provider as ColorProvider

localized = True


class Provider(ColorProvider):
    """Implement color provider for ``ru_RU`` locale."""

    all_colors = OrderedDict(
        (
            ("Античный Белый", "#FAEBD7"),
            ("Аквамарин", "#7FFFD4"),
            ("Лазурный", "#F0FFFF"),
            ("Бежевый", "#F5F5DC"),
            ("Черный", "#000000"),
            ("Синий", "#0000FF"),
            ("Сине-фиолетовый", "#8A2BE2"),
            ("Коричневый", "#A52A2A"),
            ("Шоколадный", "#D2691E"),
            ("Коралловый", "#FF7F50"),
            ("Васильковый", "#6495ED"),
            ("Малиновый", "#DC143C"),
            ("Темно-синий", "#00008B"),
            ("Темно-голубой", "#008B8B"),
            ("Темно-серый", "#A9A9A9"),
            ("Темно-зеленый", "#006400"),
            ("Темный хаки", "#BDB76B"),
            ("Темно-оранжевый", "#FF8C00"),
            ("Темно-красный", "#8B0000"),
            ("Темно-бирюзовый", "#00CED1"),
            ("Темно-фиолетовый", "#9400D3"),
            ("Темно-розовый", "#FF1493"),
            ("Тусклый серый", "#696969"),
            ("Фуксия", "#FF00FF"),
            ("Золотой", "#FFD700"),
            ("Серый", "#808080"),
            ("Зеленый", "#008000"),
            ("Желто-зеленый", "#ADFF2F"),
            ("Ярко-розовый", "#FF69B4"),
            ("Индиго", "#4B0082"),
            ("Слоновая кость", "#FFFFF0"),
            ("Хаки", "#F0E68C"),
            ("Розовато-лавандовый", "#FFF0F5"),
            ("Светло-синий", "#ADD8E6"),
            ("Светло-голубой", "#E0FFFF"),
            ("Светло-серый", "#D3D3D3"),
            ("Светло-зеленый", "#90EE90"),
            ("Светло-розовый", "#FFB6C1"),
            ("Светло-голубой", "#87CEFA"),
            ("Светло-желтый", "#FFFFE0"),
            ("Каштановый", "#800000"),
            ("Оранжевый", "#FFA500"),
            ("Оранжево-красный", "#FF4500"),
            ("Бледно-зеленый", "#98FB98"),
            ("Бледно-Бирюзовый", "#AFEEEE"),
            ("Розовый", "#FFC0CB"),
            ("Сливовый", "#DDA0DD"),
            ("Пурпурный", "#800080"),
            ("Красный", "#FF0000"),
            ("Цвет морской волны", "#2E8B57"),
            ("Серебряный", "#C0C0C0"),
            ("Бирюзовый", "#40E0D0"),
            ("Фиолетовый", "#EE82EE"),
            ("Белый", "#FFFFFF"),
            ("Желтый", "#FFFF00"),
            ("Желто-зеленый", "#9ACD32"),
        )
    )

    safe_colors = (
        "черный",
        "бордовый",
        "зеленый",
        "оливковый",
        "пурпурный",
        "teal",
        "lime",
        "синий",
        "серебряный",
        "серый",
        "желтый",
        "фуксия",
        "белый",
    )



from .. import Provider as CompanyProvider


class Provider(CompanyProvider):
    formats = (
        "{{last_name}} {{company_suffix}}",
        "{{last_name}} {{last_name}} {{company_suffix}}",
        "{{large_company}}",
    )

    large_companies = (
        "AZAL",
        "Azergold",
        "SOCAR",
        "Socar Polymer",
        "Global Export Fruits",
        "Baku Steel Company",
        "Azersun",
        "Sun Food",
        "Azərbaycan Şəkər İstehsalat Birliyi",
        "Azərsu",
        "Xəzər Dəniz Gəmiçiliyi",
        "Azərenerji",
        "Bakıelektrikşəbəkə",
        "Azəralüminium",
        "Bravo",
        "Azərpambıq Aqrar Sənaye Kompleksi",
        "CTS-Agro",
        "Azərtütün Aqrar Sənaye Kompleksi",
        "Azəripək",
        "Azfruittrade",
        "AF Holding",
        "Azinko Holding",
        "Gilan Holding",
        "Azpetrol",
        "Azərtexnolayn",
        "Bakı Gəmiqayırma Zavodu",
        "Gəncə Tekstil Fabriki",
        "Mətanət A",
        "İrşad Electronics",
    )
    company_suffixes = (
        "ASC",
        "QSC",
        "MMC",
    )

    def large_company(self) -> str:
        """
        :example: 'SOCAR'
        """
        return self.random_element(self.large_companies)



import pytest

from faker.providers.isbn import ISBN10, ISBN13
from faker.providers.isbn.en_US import Provider as ISBNProvider
from faker.providers.isbn.rules import RegistrantRule


class TestISBN10:
    def test_check_digit_is_correct(self):
        isbn = ISBN10(group="1", registrant="4516", publication="7331")
        assert isbn.check_digit == "0"
        isbn = ISBN10(group="0", registrant="06", publication="230125")
        assert isbn.check_digit == "X"
        isbn = ISBN10(group="1", registrant="4936", publication="8222")
        assert isbn.check_digit == "9"

    def test_format_length(self):
        isbn = ISBN10(group="1", registrant="4516", publication="7331")
        assert len(isbn.format()) == 10


class TestISBN13:
    def test_check_digit_is_correct(self):
        isbn = ISBN13(ean="978", group="1", registrant="4516", publication="7331")
        assert isbn.check_digit == "9"
        isbn = ISBN13(ean="978", group="1", registrant="59327", publication="599")
        assert isbn.check_digit == "0"
        isbn = ISBN13(ean="978", group="1", registrant="4919", publication="2757")
        assert isbn.check_digit == "1"

    def test_format_length(self):
        isbn = ISBN13(ean="978", group="1", registrant="4516", publication="7331")
        assert len(isbn.format()) == 13


class TestProvider:
    prov = ISBNProvider(None)

    def test_reg_pub_separation(self):
        r1 = RegistrantRule("0000000", "0000001", 1)
        r2 = RegistrantRule("0000002", "0000003", 2)
        assert self.prov._registrant_publication("00000000", [r1, r2]) == (
            "0",
            "0000000",
        )
        assert self.prov._registrant_publication("00000010", [r1, r2]) == (
            "0",
            "0000010",
        )
        assert self.prov._registrant_publication("00000019", [r1, r2]) == (
            "0",
            "0000019",
        )
        assert self.prov._registrant_publication("00000020", [r1, r2]) == (
            "00",
            "000020",
        )
        assert self.prov._registrant_publication("00000030", [r1, r2]) == (
            "00",
            "000030",
        )
        assert self.prov._registrant_publication("00000031", [r1, r2]) == (
            "00",
            "000031",
        )
        assert self.prov._registrant_publication("00000039", [r1, r2]) == (
            "00",
            "000039",
        )

    def test_rule_not_found(self):
        with pytest.raises(Exception):
            r = RegistrantRule("0000000", "0000001", 1)
            self.prov._registrant_publication("0000002", [r])
